package jsetl.test;
import jsetl.*;
import jsetl.exception.Failure;
import jsetl.IntLVar;
import jsetl.Solver;
import jsetl.exception.InitLObjectException;
import jsetl.test.utilities.comparing.ConstraintStringComparator;
import junit.framework.TestCase;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import static org.junit.jupiter.api.Assertions.*;

import java.io.IOException;
import java.util.HashSet;
import java.util.Set;

/**
 * Tests for Ris (in particular constraints over them and logical sets).
 * @author Andrea Fois
 */

public class RisTest extends TestCase{
	
	@Before
	public void setUp() {
	    LObject.USEJOLLYTOKEN = true;
    }
	
	@After
	public void tearDown(){LObject.USEJOLLYTOKEN = false;}

    @Test
    public void testRisTestContains(){
        IntLVar x = new IntLVar();
        IntLSet d = new IntLSet(1,4);
        Ris ris = new Ris(x, d, x.neq(2), x.sum(1)); // {2,4,5}
        assertFalse(ris.testContains(1));
        assertTrue(ris.testContains(2));
        assertFalse(ris.testContains(3));
        assertTrue(ris.testContains(4));
        assertTrue(ris.testContains(5));
        assertFalse(ris.testContains(6));
    }

    @Test
    public void testRisNormalizeSet(){
        IntLVar x = new IntLVar();
        IntLSet d = new IntLSet();
        Ris ris = new Ris(x, d, x.neq(2), x.sum(1)); // {2,4,5}
        assertThrows(UnsupportedOperationException.class, () -> ris.normalizeSet());
    }

    @Test
    public void testRisInTrue1(){
	    Ris ris = new Ris(new LVar(), new LSet(), Constraint.truec());
	    LSet lSet = LSet.empty().ins(1, ris, 3);
	    assertTrue(ris.in(lSet).check());
    }

    @Test
    public void testRisInTrue2(){
        Ris ris = new Ris(new LVar(), new LSet(), Constraint.truec());
        LSet lSet = new LSet().ins(1, 2, 3);
        assertTrue(ris.in(lSet).check());
    }

    @Test
    public void testRisInFalse(){
        Ris ris = new Ris(new LVar(), new LSet(), Constraint.truec());
        LSet lSet = LSet.empty().ins(1, 2, 3);
        assertFalse(ris.in(lSet).check());
    }

    @Test
    public void testRisNinTrue(){
        Ris ris = new Ris(new LVar(), new LSet(), Constraint.truec());
        LSet lSet = new LSet().ins(1,2,3);
        assertTrue(ris.nin(lSet).check());
    }

    @Test
    public void testRisNinFalse(){
        Ris ris = new Ris(new LVar(), new LSet(), Constraint.truec());
        LSet lSet = new LSet().ins(1,ris,3);
        assertFalse(ris.nin(lSet).check());
    }

	@Test
    public void testRisWithComplexLListControlTerm(){
	    LVar x1 = new LVar(), x2 = new LVar(), x3 = new LVar(), x4 = new LVar();
	    LList controlTerm = new LList().ins(x1,x2,x3,x4);
	    LSet domain = new LSet();
	    Ris ris = new Ris(controlTerm, domain, x1.in(domain).and(x2.neq(x3)).and(x1.eq(x4)), LList.empty().ins(x1,x2,x3));

        LList l1 = new LList().ins(1,2,2);
        assertFalse(ris.contains(l1).check());

        LList l2 = new LList().ins(1,2,3);
        assertTrue(ris.contains(l2).check());
        assertEquals("{[1,2,3,1|_?],1/_?}", domain.toString());

        LList l3 = LList.empty().ins(2,3,4);
        assertTrue(ris.contains(l3).check());
        assertEquals("{[1,2,3,1|_?],1,[2,3,4,2|_?],2/_?}", domain.toString());
        for(Object o : domain.getValue()){
            if(o instanceof LList) {
                assertTrue(ris.contains(new LList().ins(o, 2, 3)).check());
                domain.output();
            }
        }

    }


	/*
	 -------------------------------------------------------------------------------------------------------------
          ______ ____  _    _         _      _____ _________     __  _______ ______  _____ _______ _____
         |  ____/ __ \| |  | |  /\   | |    /_   _/__   __\ \   / / /__   __|  ____|/ ____/__   __/ ____|
         | /__ | |  | | |  | | /  \  | |      | |    | |   \ \_/ /     | |  | /__  | (___    | | | (___
         |  __|| |  | | |  | |/ /\ \ | |      | |    | |    \   /      | |  |  __|  \___ \   | |  \___ \
         | /___| /__| | /__| / ____ \| /____ _| /_   | |     | |       | |  | /____ ____) |  | |  ____) |
         /______\___\_\\____/_/    \_\______/_____|  /_|     /_|       /_|  /______/_____/   /_| /_____/
     -------------------------------------------------------------------------------------------------------------
	*/

	@Test
    public void testInitClosedRisEmptySetFalse01(){
	    LSet empty = LSet.empty();
	    Ris ris = new Ris(new LVar(), LSet.mkSet(3), Constraint.truec());

	    Solver solver = new Solver();
	    solver.add(ris.eq(empty));

	    assertFalse(solver.check());
    }

	@Test
    public void testEmptySetEmptyRis(){
	    LSet A = LSet.empty();
	    LSet D = LSet.empty();
	    Ris B = new Ris(new LVar(), D, Constraint.truec());

	    Solver solver = new Solver();
	    solver.add(A.eq(B));

	    assertEquals(true, solver.check());
	    assertEquals("[]", solver.getConstraint().toString());
	    assertEquals(false, solver.nextSolution());
    }
	
	@Test
	public void testEqEmptysetInitRisTrue1() {
		LSet A = LSet.empty();
		LSet D = LSet.empty().ins(1).ins(2);
		LVar x = new LVar("x");
		Ris B = new Ris(x, D, x.neq(1).and(x.neq(2)));

		Solver solver = new Solver();
		solver.add(A.eq(B));
		assertEquals(true, solver.check());
		assertEquals("[]", solver.getConstraint().toString());
        assertEquals(false, solver.nextSolution());
	}

    @Test
    public void testEqEmptysetInitRisTrue2() {
        LSet A = LSet.empty();
        LSet D = LSet.empty().ins(1).ins(2);
        IntLVar x = new IntLVar("x");
        Ris B = new Ris(x, D, x.ge(3));

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(true, solver.check());
        assertEquals("[]", solver.getConstraint().toString());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqEmptysetInitRisTrue3() {
        LSet A = LSet.empty();
        LSet D = new LSet().ins(1).ins(2);
        LVar x = new LVar("x");
        Ris B = new Ris(x, D, x.neq(1).and(x.neq(2)));

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(true, solver.check());
        assertEquals("{} = { _x : _? | _x neq 1 AND _x neq 2 @ _x }", solver.getConstraint().toString());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqEmptysetInitRisTrue4() {
        LSet A = LSet.empty();
        LSet D = new LSet().ins(1).ins(2);
        IntLVar x = new IntLVar("x");
        Ris B = new Ris(x, D, x.ge(3));

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(true, solver.check());
        assertEquals("{} = { _x : _? | _x >= 3 @ _x }", solver.getConstraint().toString());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqEmptysetInitRisFalse1() {
        LSet A = LSet.empty();
        LSet D = LSet.empty().ins(1).ins(2);
        LVar x = new LVar("x");
        Ris B = new Ris(x, D, x.neq(1).and(x.neq(3)));

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(false, solver.check());
    }

    @Test
    public void testEqEmptysetInitRisFalse2() {
        LSet A = LSet.empty();
        LSet D = LSet.empty().ins(1).ins(2);
        IntLVar x = new IntLVar("x");
        Ris B = new Ris(x, D, x.lt(2));

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(false, solver.check());
    }

    @Test
    public void testEqEmptysetInitRisFalse3() {
        LSet A = LSet.empty();
        LSet D = new LSet().ins(1).ins(2);
        LVar x = new LVar("x");
        Ris B = new Ris(x, D, x.neq(1).and(x.neq(3)));

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(false, solver.check());
    }

    @Test
    public void testEqEmptysetInitRisFalse4() {
        LSet A = LSet.empty();
        LSet D = new LSet().ins(1).ins(2);
        IntLVar x = new IntLVar("x");
        Ris B = new Ris(x, D, x.lt(2));

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(false, solver.check());
    }

    @Test
    public void testEqEmptysetNotInitRisTrue() {
        LSet A = LSet.empty();
        LSet D = new LSet();
        Ris B = new Ris(new LVar(), D, Constraint.truec());

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(true, solver.check());
        assertEquals("{} = { _? : _? | true @ _? }", solver.getConstraint().toString());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqInitSetEmptyRisFalse01() {
        LSet A = LSet.empty().ins("a");
        LSet D = LSet.empty();
        Ris B = new Ris(new LVar(), D, Constraint.truec());

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(false, solver.check());
    }

    @Test
    public void testEqInitSetEmptyRisFalse02() {
        LSet A = new LSet().ins("a");
        LSet D = LSet.empty();
        Ris B = new Ris(new LVar(), D, Constraint.truec());

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(false, solver.check());
    }

    @Test
    public void testEqInitSetInitRisTrue01() {
        LSet A = LSet.empty().ins("a").ins("b").ins("c");
        LSet D = LSet.empty().ins("c").ins("a").ins("b");
        Ris B = new Ris(new LVar(), D, Constraint.truec());

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(true, solver.check());
        assertEquals(false, solver.nextSolution());



    }



    @Test
    public void testEqInitSetInitRisTrue02() {
        LSet A = new LSet("A'").ins("a").ins("b").ins("c");
        LSet D = new LSet("D").ins("d");
        Ris B = new Ris(new LVar(), D, Constraint.truec());

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(true, solver.check());
        assertEquals("[]", solver.getConstraint().toString());
        assertEquals("{d,c,b,a/_?}", D.toString());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqInitSetInitRisTrue03() {
        IntLSet A = IntLSet.empty().ins(1).ins(2).ins(3);
        IntLSet D = new IntLSet("D").ins(0).ins(1).ins(2).ins(3);
        IntLVar x = new IntLVar();
        Ris B = new Ris(x, D, x.lt(3), x.sum(1));

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(true, solver.check());
        ConstraintStringComparator constraintStringComparator = new ConstraintStringComparator(
                "{} = { _? : _D | _? < 3 @ _?, _? = _? + 1 }",
                solver.getConstraint()
        );
        assertTrue(constraintStringComparator.areEqualInSameOrder());
        assertEquals("{0..3/_D}", D.toString());
    }

    @Test
    public void testEqInitSetInitRisTrue04() {
        IntLSet A = IntLSet.empty().ins(1).ins(2).ins(3);
        IntLVar a = new IntLVar();
        IntLSet D = new IntLSet("D").ins(0).ins(1).ins(a).ins(3);
        IntLVar x = new IntLVar();
        Ris B = new Ris(x, D, x.lt(3), x.sum(1));

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(true, solver.check());
        ConstraintStringComparator constraintStringComparator = new ConstraintStringComparator(
                "{ _? : _D | _? < 3 @ _?, _? = _? + 1 } = {}",
                solver.getConstraint()
        );
        assertTrue(constraintStringComparator.areEqualInSameOrder());
        assertEquals("{0..3/_D}", D.toString());
        assertEquals("2", a.toString());
    }

    @Test
    public void testEqInitSetInitRisTrue05() {
        LSet A = new LSet().ins("a").ins("b").ins("c");
        LSet D = new LSet("D").ins("d").ins("e");
        Ris B = new Ris(new LVar(), D, Constraint.truec());

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(true, solver.check());
        assertEquals("{e,d,c,b,a/_?}", D.toString());
        assertEquals("{c,b,a,e,d/{ _? : _? | true @ _? }}", A.toString());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqInitSetInitRisTrue06() {
        LSet A = LSet.empty().ins("a").ins("b").ins("c");
        LVar a = new LVar();
        LVar b = new LVar();
        LVar c = new LVar();
        LSet D = LSet.empty().ins(a).ins(b).ins(c);
        Ris B = new Ris(new LVar(), D, Constraint.truec());

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(true, solver.check());
        assertEquals("{c,b,a}", D.toString());
        assertEquals(true, solver.nextSolution());
        assertEquals("{c,a,b}", D.toString());
        assertEquals(true, solver.nextSolution());
        assertEquals("{b,c,a}", D.toString());
        assertEquals(true, solver.nextSolution());
        assertEquals("{b,a,c}", D.toString());
        assertEquals(true, solver.nextSolution());
        assertEquals("{a,c,b}", D.toString());
        assertEquals(true, solver.nextSolution());
        assertEquals("{a,b,c}", D.toString());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqInitSetInitRisFalse01() {
        LSet A = LSet.empty().ins("a").ins("b").ins("c");
        LSet D = new LSet("D");
        LVar x = new LVar();
        Ris B = new Ris(x, D, x.neq("b"));

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(false, solver.check());
    }

    @Test
    public void testEqInitSetInitRisFalse02() {
        LSet A = LSet.empty().ins("a").ins("b").ins("c");
        LSet D = new LSet("D").ins("d");
        Ris B = new Ris(new LVar(), D, Constraint.truec());

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(false, solver.check());
    }

    @Test
    public void testEqInitSetNotInitRisTrue01() {
        LSet A = new LSet().ins("a").ins("b").ins("c");
        LSet D = new LSet("D");
        Ris B = new Ris(new LVar(), D, Constraint.truec());

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(true, solver.check());
        assertEquals("{c,b,a/_?}", D.toString());
        assertEquals("{c,b,a/{ _? : _? | true @ _? }}", A.toString());
        assertEquals("[]", solver.getConstraint().toString());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqInitSetNotInitRisTrue02() {
        LSet A = LSet.empty().ins("a").ins("b").ins("c");
        LSet D = new LSet("D");
        Ris B = new Ris(new LVar(), D, Constraint.truec());

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(true, solver.check());
        assertEquals("{c,b,a/_?}", D.toString());
        assertEquals("{c,b,a}", A.toString());
        assertEquals("{ _? : _? | true @ _? } = {}", solver.getConstraint().toString());

        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqInitSetNotInitRisTrue03() {
        LSet A = LSet.empty().ins(1).ins(2).ins(3);
        LSet D = new LSet("D");
        IntLVar x = new IntLVar();
        Ris B = new Ris(x, D, x.lt(7), x.sub(2));

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(true, solver.check());
        assertEquals("{5,4,3/_?}", D.toString());
        ConstraintStringComparator constraintStringComparator = new ConstraintStringComparator(
                "{ _? : _? | _? < 7 @ _?, _? = _? - 2 } = {}",
                solver.getConstraint()
        );
        assertTrue(constraintStringComparator.areEqualInSameOrder());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqInitSetNotInitRisFalse01() {
        LSet A = LSet.empty().ins("a").ins("b").ins("c");
        LSet D = new LSet("D").ins("d");
        LVar x = new LVar("x");
        Ris B = new Ris(x, D, x.neq("b"));

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(false, solver.check());
    }

    @Test
    public void testEqInitSetNotInitRisFalse02() {
        LSet A = new LSet().ins("a").ins("b").ins("c");
        LSet D = new LSet("D").ins("d");
        LVar x = new LVar("x");
        Ris B = new Ris(x, D, x.neq("b"));

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(false, solver.check());
    }

    @Test
    public void testEqInitSetNotInitRisFalse03() {
        IntLSet A = new IntLSet().ins(1).ins(3).ins(2);
        IntLSet D = new IntLSet("D").ins(0);
        IntLVar x = new IntLVar("x");
        Ris B = new Ris(x, D, x.lt(1), x.sum(2));

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(false, solver.check());
    }

    @Test
    public void testEqNotInitSetEmptyRisTrue() {
        LSet A = new LSet("A");
        LSet D = LSet.empty();
        Ris B = new Ris(new LVar(), D, Constraint.truec());

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(true, solver.check());
        assertEquals("{}", A.toString());
        assertEquals("[]", solver.getConstraint().toString());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqNotInitSetInitRisTrue() {
        LSet A = new LSet("A");
        LSet D = new LSet().ins("a");
        Ris B = new Ris(new LVar(), D, Constraint.truec());

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(true, solver.check());
        assertEquals("{a/{ _? : _? | true @ _? }}", A.toString());
        assertEquals("[]", solver.getConstraint().toString());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqNotInitSetNotInitRisTrue() {
        LSet A = new LSet("A");
        LSet D = new LSet("D");
        LVar x = new LVar();
        Ris B = new Ris(x, D, x.neq("a"));

        Solver solver = new Solver();
        solver.add(A.eq(B));

        assertEquals(true, solver.check());
        assertEquals("{ _? : _D | _? neq a @ _? }", A.toString());
        assertEquals("[]", solver.getConstraint().toString());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEmptyRisEmptySet(){
        LSet A = LSet.empty();
        LSet D = LSet.empty();
        Ris B = new Ris(new LVar(), D, Constraint.truec());

        Solver solver = new Solver();
        solver.add(B.eq(A));

        assertEquals(true, solver.check());
        assertEquals("[]", solver.getConstraint().toString());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqEmptyRisInitSetFalse01() {
        LSet A = LSet.empty().ins("a");
        LSet D = LSet.empty();
        Ris B = new Ris(new LVar(), D, Constraint.truec());

        Solver solver = new Solver();
        solver.add(B.eq(A));
        assertEquals(false, solver.check());
    }

    @Test
    public void testEqEmptyRisInitSetFalse02() {
        LSet A = new LSet().ins("a");
        LSet D = LSet.empty();
        Ris B = new Ris(new LVar(), D, Constraint.truec());

        Solver solver = new Solver();
        solver.add(B.eq(A));
        assertEquals(false, solver.check());
    }

    @Test
    public void testEqEmptyRisNotInitSetTrue() {
        LSet A = new LSet("A");
        LSet D = LSet.empty();
        Ris B = new Ris(new LVar(), D, Constraint.truec());

        Solver solver = new Solver();
        solver.add(B.eq(A));
        assertEquals(true, solver.check());
        assertEquals("{}", A.toString());
        assertEquals("[]", solver.getConstraint().toString());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqEmptyRisEmptyRisTrue() {
        Ris A = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        Ris B = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(true, solver.check());
        assertEquals("{ _? : {} | true @ _? }", A.toString());
        assertEquals("[]", solver.getConstraint().toString());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqEmptyRisInitRisFALSE() {
        Ris A = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        Ris B = new Ris(new LVar(), new LSet().ins("a"), Constraint.truec());

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(false, solver.check());
    }

    @Test
    public void testEqEmptyRisInitRisTRUE1() {
        Ris A = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        LVar x = new LVar();
        Ris B = new Ris(x, new LSet().ins("a"), x.neq("a"));

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(true, solver.check());
        assertEquals("{} = { _? : _? | _? neq a @ _? }", solver.getConstraint().toString());
    }

    @Test
    public void testEqEmptyRisInitRisTRUE2() {
        Ris A = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        LVar x = new LVar();
        Ris B = new Ris(x, LSet.empty().ins("a"), x.neq("a"));

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(true, solver.check());
        assertEquals("[]", solver.getConstraint().toString());
    }

    @Test
    public void testEqEmptyRisNotInitRisTRUE() {
        Ris A = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        Ris B = new Ris(new LVar(), new LSet(), Constraint.truec());

        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(true, solver.check());
        assertEquals("{} = { _? : _? | true @ _? }", solver.getConstraint().toString());
    }

    @Test
    public void testEqInitRisEmptysetTrue1() {
        LSet A = LSet.empty();
        LSet D = LSet.empty().ins(1).ins(2);
        LVar x = new LVar("x");
        Ris B = new Ris(x, D, x.neq(1).and(x.neq(2)));

        Solver solver = new Solver();
        solver.add(B.eq(A));
        assertEquals(true, solver.check());
        assertEquals("[]", solver.getConstraint().toString());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqInitRisEmptysetTrue2() {
        LSet A = LSet.empty();
        LSet D = LSet.empty().ins(1).ins(2);
        IntLVar x = new IntLVar("x");
        Ris B = new Ris(x, D, x.ge(3));

        Solver solver = new Solver();
        solver.add(B.eq(A));
        assertEquals(true, solver.check());
        assertEquals("[]", solver.getConstraint().toString());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqInitRisEmptysetTrue3() {
        LSet A = LSet.empty();
        LSet D = new LSet().ins(1).ins(2);
        LVar x = new LVar("x");
        Ris B = new Ris(x, D, x.neq(1).and(x.neq(2)));

        Solver solver = new Solver();
        solver.add(B.eq(A));
        assertEquals(true, solver.check());
        assertEquals("{ _x : _? | _x neq 1 AND _x neq 2 @ _x } = {}", solver.getConstraint().toString());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqInitRisEmptysetTrue4() {
        LSet A = LSet.empty();
        LSet D = new LSet().ins(1).ins(2);
        IntLVar x = new IntLVar("x");
        Ris B = new Ris(x, D, x.ge(3));

        Solver solver = new Solver();
        solver.add(B.eq(A));
        assertEquals(true, solver.check());
        assertEquals("{ _x : _? | _x >= 3 @ _x } = {}", solver.getConstraint().toString());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqInitRisEmptysetFalse1() {
        LSet A = LSet.empty();
        LSet D = LSet.empty().ins(1).ins(2);
        LVar x = new LVar("x");
        Ris B = new Ris(x, D, x.neq(1).and(x.neq(3)));

        Solver solver = new Solver();
        solver.add(B.eq(A));
        assertEquals(false, solver.check());
    }

    @Test
    public void testEqInitRisEmptysetFalse2() {
        LSet A = LSet.empty();
        LSet D = LSet.empty().ins(1).ins(2);
        IntLVar x = new IntLVar("x");
        Ris B = new Ris(x, D, x.lt(2));

        Solver solver = new Solver();
        solver.add(B.eq(A));
        assertEquals(false, solver.check());
    }

    @Test
    public void testEqInitRisEmptysetFalse3() {
        LSet A = LSet.empty();
        LSet D = new LSet().ins(1).ins(2);
        LVar x = new LVar("x");
        Ris B = new Ris(x, D, x.neq(1).and(x.neq(3)));

        Solver solver = new Solver();
        solver.add(B.eq(A));
        assertEquals(false, solver.check());
    }

    @Test
    public void testEqInitRisEmptysetFalse4() {
        LSet A = LSet.empty();
        LSet D = new LSet().ins(1).ins(2);
        IntLVar x = new IntLVar("x");
        Ris B = new Ris(x, D, x.lt(2));

        Solver solver = new Solver();
        solver.add(B.eq(A));
        assertEquals(false, solver.check());
    }

    @Test
    public void testEqInitRisInitSetTrue01() {
        LSet A = LSet.empty().ins("a").ins("b").ins("c");
        LSet D = LSet.empty().ins("c").ins("a").ins("b");
        Ris B = new Ris(new LVar(), D, Constraint.truec());

        Solver solver = new Solver();
        solver.add(B.eq(A));
        assertEquals(true, solver.check());
        assertEquals(false, solver.nextSolution());



    }



    @Test
    public void testEqInitRisInitSetTrue02() {
        LSet A = new LSet("A'").ins("a").ins("b").ins("c");
        LSet D = new LSet("D").ins("d");
        Ris B = new Ris(new LVar(), D, Constraint.truec());

        Solver solver = new Solver();
        solver.add(B.eq(A));
        assertEquals(true, solver.check());
        assertEquals("[]", solver.getConstraint().toString());
        assertEquals("{d,c,b,a/_?}", D.toString());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqInitRisInitSetTrue03() {
        IntLSet A = IntLSet.empty().ins(1).ins(2).ins(3);
        IntLSet D = new IntLSet("D").ins(0).ins(1).ins(2).ins(3);
        IntLVar x = new IntLVar();
        Ris B = new Ris(x, D, x.lt(3), x.sum(1));

        Solver solver = new Solver();
        solver.add(B.eq(A));
        assertEquals(true, solver.check());
        ConstraintStringComparator constraintStringComparator = new ConstraintStringComparator(
                "{ _? : _D | _? < 3 @ _?, _? = _? + 1 } = {}",
                solver.getConstraint()
        );
        assertTrue(constraintStringComparator.areEqualInSameOrder());
        assertEquals("{0..3/_D}", D.toString());
    }

    @Test
    public void testEqInitRisInitSetTrue04() {
        IntLSet A = IntLSet.empty().ins(1).ins(2).ins(3);
        IntLVar a = new IntLVar();
        IntLSet D = new IntLSet("D").ins(0).ins(1).ins(a).ins(3);
        IntLVar x = new IntLVar();
        Ris B = new Ris(x, D, x.lt(3), x.sum(1));

        Solver solver = new Solver();
        solver.add(B.eq(A));
        assertEquals(true, solver.check());
        ConstraintStringComparator constraintStringComparator = new ConstraintStringComparator(
                "{ _? : _D | _? < 3 @ _?, _? = _? + 1 } = {}",
                solver.getConstraint()
        );
        assertTrue(constraintStringComparator.areEqualInSameOrder());
        assertEquals("{0..3/_D}", D.toString());
        assertEquals("2", a.toString());
    }

    @Test
    public void testEqInitRisInitSetTrue05() {
        LSet A = new LSet().ins("a").ins("b").ins("c");
        LSet D = new LSet("D").ins("d");
        Ris B = new Ris(new LVar(), D, Constraint.truec());

        Solver solver = new Solver();
        solver.add(B.eq(A));
        assertEquals(true, solver.check());
        assertEquals("{d,c,b,a/_?}", D.toString());
        assertEquals("{c,b,a,d/{ _? : _? | true @ _? }}", A.toString());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqInitRisInitSetTrue06() {
        LSet A = LSet.empty().ins("a").ins("b").ins("c");
        LVar a = new LVar();
        LVar b = new LVar();
        LVar c = new LVar();
        LSet D = LSet.empty().ins(a).ins(b).ins(c);
        Ris B = new Ris(new LVar(), D, Constraint.truec());

        Solver solver = new Solver();
        solver.add(B.eq(A));
        assertEquals(true, solver.check());
        assertEquals("{c,b,a}", D.toString());
        assertEquals(true, solver.nextSolution());
        assertEquals("{c,a,b}", D.toString());
        assertEquals(true, solver.nextSolution());
        assertEquals("{b,c,a}", D.toString());
        assertEquals(true, solver.nextSolution());
        assertEquals("{b,a,c}", D.toString());
        assertEquals(true, solver.nextSolution());
        assertEquals("{a,c,b}", D.toString());
        assertEquals(true, solver.nextSolution());
        assertEquals("{a,b,c}", D.toString());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqInitRisInitSetFalse01() {
        LSet A = LSet.empty().ins("a").ins("b").ins("c");
        LSet D = new LSet("D");
        LVar x = new LVar();
        Ris B = new Ris(x, D, x.neq("b"));

        Solver solver = new Solver();
        solver.add(B.eq(A));
        assertEquals(false, solver.check());
    }

    @Test
    public void testEqInitRisInitSetFalse02() {
        LSet A = LSet.empty().ins("a").ins("b").ins("c");
        LSet D = new LSet("D").ins("d");
        Ris B = new Ris(new LVar(), D, Constraint.truec());

        Solver solver = new Solver();
        solver.add(B.eq(A));
        assertEquals(false, solver.check());
    }

    @Test
    public void testEqInitRisNotInitSetTrue() {
        LSet A = new LSet("A");
        LSet D = new LSet().ins("a");
        Ris B = new Ris(new LVar(), D, Constraint.truec());

        Solver solver = new Solver();
        solver.add(B.eq(A));
        assertEquals(true, solver.check());
        assertEquals("{a/{ _? : _? | true @ _? }}", A.toString());
        assertEquals("[]", solver.getConstraint().toString());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqInitRisInitRisTrue1(){
	    LSet D = new LSet().ins("a");
	    LVar x = new LVar();
	    Ris A = new Ris(x, D, x.neq("b"));
	    LSet E = new LSet().ins("a").ins("c");
	    LVar y = new LVar();
	    Ris B = new Ris(y, E, y.neq("d"));
	    Solver solver = new Solver();
	    solver.add(A.eq(B));
	    assertEquals(true, solver.check());
	    assertEquals("{a,c/_?}", D.toString());
	    assertEquals("{c,a/_?}", E.toString());
	    assertEquals("{ _? : _? | _? neq b @ _? } = { _? : _? | _? neq d @ _? }", solver.getConstraint().toString());
	    assertEquals(true, solver.nextSolution());
        assertEquals("{a,c/_?}", D.toString());
        assertEquals("{c,a/_?}", E.toString());
        assertEquals("{ _? : _? | _? neq d @ _? } = { _? : _? | _? neq b @ _? }", solver.getConstraint().toString());
        assertEquals(true, solver.nextSolution());
        assertEquals("{a,c/_?}", D.toString());
        assertEquals("{c,a/_?}", E.toString());
        assertEquals("{ _? : _? | _? neq b @ _? } = { _? : _? | _? neq d @ _? }", solver.getConstraint().toString());
    }

    @Test
    public void testEqInitRisInitRisTrue2() {
        LSet D = LSet.empty().ins("a");
        LVar x = new LVar();
        Ris A = new Ris(x, D, x.neq("b"));
        LSet E = new LSet().ins("a").ins("c");
        LVar y = new LVar();
        Ris B = new Ris(y, E, y.neq("c"));
        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(true, solver.check());
        assertEquals("{a}", D.toString());
        assertEquals("{c,a/_?}", E.toString());
        assertEquals("{} = { _? : _? | _? neq c @ _? }", solver.getConstraint().toString());
        assertEquals(true, solver.nextSolution());
        assertEquals("{a}", D.toString());
        assertEquals("{c,a/_?}", E.toString());
        assertEquals("{ _? : _? | _? neq c @ _? } = {}", solver.getConstraint().toString());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqInitRisInitRisTrue3() {
        LSet D = LSet.empty().ins("a");
        LVar x = new LVar();
        Ris A = new Ris(x, D, x.neq("b"));
        LSet E = LSet.empty().ins("a").ins("c");
        LVar y = new LVar();
        Ris B = new Ris(y, E, y.neq("c"));
        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(true, solver.check());
        assertEquals("{a}", D.toString());
        assertEquals("{c,a}", E.toString());
        assertEquals("[]", solver.getConstraint().toString());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqInitRisInitRisTrue4() {
        IntLSet D = IntLSet.empty().ins(3);
        IntLVar x = new IntLVar();
        Ris A = new Ris(x, D, x.mod(2).eq(1), x.sum(1));
        IntLSet E = IntLSet.empty().ins(2).ins(1);
        IntLVar y = new IntLVar();
        Ris B = new Ris(y, E, y.mod(2).eq(0), y.sum(2));
        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(true, solver.check());
        assertEquals("{3}", D.toString());
        assertEquals("{1..2}", E.toString());
        assertEquals("[]", solver.getConstraint().toString());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqInitRisInitRisTrue5() {
        IntLSet D = IntLSet.empty().ins(3);
        IntLVar x = new IntLVar();
        Ris A = new Ris(x, D, x.mod(2).eq(1), x.sum(1));
        IntLSet E = new IntLSet().ins(2).ins(1);
        IntLVar y = new IntLVar();
        Ris B = new Ris(y, E, y.mod(2).eq(0), y.sum(2));
        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(true, solver.check());
        assertEquals("{3}", D.toString());
        assertEquals("{1..2/_?}", E.toString());
        ConstraintStringComparator constraintStringComparator1 = new ConstraintStringComparator(
                "{} = { _? : _? | _? = 0 AND _? = _? mod 2 @ _?, _? = _? + 2 }",
                solver.getConstraint()
        );

        assertEquals(true, constraintStringComparator1.areEqualInSameOrder());
        assertEquals(true, solver.nextSolution());
        assertEquals("{3}", D.toString());
        assertEquals("{1..2/_?}", E.toString());
        ConstraintStringComparator constraintStringComparator2 = new ConstraintStringComparator(
                "{ _? : _? | _? = 0 AND _? = _? mod 2 @ _?, _? = _? + 2 } = {}",
                solver.getConstraint()
        );
        assertEquals(true, constraintStringComparator2.areEqualInSameOrder());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqInitRisInitRisTrue6() {
        IntLSet D = new IntLSet().ins(3);
        IntLVar x = new IntLVar();
        Ris A = new Ris(x, D, x.mod(2).eq(1), x.sum(1));
        IntLSet E = new IntLSet().ins(2).ins(1);
        IntLVar y = new IntLVar();
        Ris B = new Ris(y, E, y.mod(2).eq(0), y.sum(2));
        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(true, solver.check());
        assertEquals("{3/_?}", D.toString());
        assertEquals("{1..2/_?}", E.toString());
        ConstraintStringComparator constraintStringComparator = new ConstraintStringComparator(
                "{ _? : _? | _? = 1 AND _? = _? mod 2 @ _?, _? = _? + 1 } = { _? : _? | _? = 0 AND _? = _? mod 2 @ _?, _? = _? + 2 }",
                solver.getConstraint()
        );
        assertTrue(constraintStringComparator.areEqualInSameOrder());
        assertEquals(true, solver.nextSolution());
        assertEquals("{3/_?}", D.toString());
        assertEquals("{1..2/_?}", E.toString());
        constraintStringComparator = new ConstraintStringComparator(
                "{ _? : _? | _? = 0 AND _? = _? mod 2 @ _?, _? = _? + 2 } = { _? : _? | _? = 1 AND _? = _? mod 2 @ _?, _? = _? + 1 }",
                solver.getConstraint()
        );
        assertTrue(constraintStringComparator.areEqualInSameOrder());
        assertEquals(true, solver.nextSolution());
        assertEquals("{3/_?}", D.toString());
        assertEquals("{1..2/_?}", E.toString());

        constraintStringComparator = new ConstraintStringComparator(
                "{ _? : _? | _? = 1 AND _? = _? mod 2 @ _?, _? = _? + 1 } = { _? : _? | _? = 0 AND _? = _? mod 2 @ _?, _? = _? + 2 }",
                solver.getConstraint()
        );
        assertTrue(constraintStringComparator.areEqualInSameOrder());
        if(solver.getOptimizationOptions().areSetUnificationOptimizationsEnabled())
            return;
        assertEquals(true, solver.nextSolution());
        assertEquals("{3/_?}", D.toString());
        assertEquals("{1..2/_?}", E.toString());
        constraintStringComparator = new ConstraintStringComparator(
                "{ _? : _? | _? = 0 AND _? = _? mod 2 @ _?, _? = _? + 2 } = { _? : _? | _? = 1 AND _? = _? mod 2 @ _?, _? = _? + 1 }",
                solver.getConstraint()
        );
        assertTrue(constraintStringComparator.areEqualInSameOrder());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqInitRisInitRisTrue7() {
        IntLSet D = new IntLSet().ins(3);
        IntLVar x = new IntLVar();
        Ris A = new Ris(x, D, x.mod(2).eq(1), x.sum(1));
        IntLSet E = IntLSet.empty().ins(2).ins(1).ins(0).ins(4);
        IntLVar y = new IntLVar();
        Ris B = new Ris(y, E, y.mod(2).eq(0), y.sum(2));
        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(true, solver.check());
        assertEquals("{1, 3, 5/_?}", D.toString());
        assertEquals("{0..2, 4}", E.toString());
        ConstraintStringComparator constraintStringComparator = new ConstraintStringComparator(
                "{ _? : _? | _? = 1 AND _? = _? mod 2 @ _?, _? = _? + 1 } = {}",
                solver.getConstraint()
        );
        assertTrue(constraintStringComparator.areEqualInSameOrder());
        assertEquals(true, solver.nextSolution());
        assertEquals("{1, 3, 5/_?}", D.toString());
        assertEquals("{0..2, 4}", E.toString());
        constraintStringComparator = new ConstraintStringComparator(
                "{ _? : _? | _? = 1 AND _? = _? mod 2 @ _?, _? = _? + 1 } = {}",
                solver.getConstraint()
        );
        assertTrue(constraintStringComparator.areEqualInSameOrder());
        assertEquals(false, solver.nextSolution());
	}

    @Test
    public void testEqInitRisInitRisFalse1() {
        LSet D = new LSet().ins("a");
        LVar x = new LVar();
        Ris A = new Ris(x, D, x.neq("c"));
        LSet E = new LSet().ins("a").ins("c");
        LVar y = new LVar();
        Ris B = new Ris(y, E, y.neq("d"));
        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(false, solver.check());
    }

    @Test
    public void testEqInitRisInitRisFalse2(){
        LSet D = LSet.empty().ins("a");
        LVar x = new LVar();
        Ris A = new Ris(x, D, x.neq("b"));
        LSet E = new LSet().ins("a").ins("c");
        LVar y = new LVar();
        Ris B = new Ris(y, E, y.neq("d"));
        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(false, solver.check());
	}

    @Test
    public void testEqInitRisInitRisFalse3() {
        LSet D = LSet.empty().ins("a");
        LVar x = new LVar();
        Ris A = new Ris(x, D, x.neq("b"));
        LSet E = LSet.empty().ins("a").ins("c");
        LVar y = new LVar();
        Ris B = new Ris(y, E, y.neq("d"));
        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(false, solver.check());
    }

    @Test
    public void testEqInitRisInitRisFalse4() {
        IntLSet D = IntLSet.empty().ins(3);
        IntLVar x = new IntLVar();
        Ris A = new Ris(x, D, x.mod(2).eq(1), x.sum(2));
        IntLSet E = IntLSet.empty().ins(2).ins(1);
        IntLVar y = new IntLVar();
        Ris B = new Ris(y, E, y.mod(2).eq(0), y.sum(2));
        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(false, solver.check());
	}

    @Test
    public void testEqInitRisInitRisFalse5()  {
        IntLSet D = IntLSet.empty().ins(3);
        IntLVar x = new IntLVar();
        Ris A = new Ris(x, D, x.mod(2).eq(1), x.sum(2));
        IntLSet E = new IntLSet().ins(2).ins(1);
        IntLVar y = new IntLVar();
        Ris B = new Ris(y, E, y.mod(2).eq(0), y.sum(2));
        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(false, solver.check());
	}

    @Test
    public void testEqInitRisInitRisFalse6() {
        IntLSet D = new IntLSet().ins(3);
        IntLVar x = new IntLVar();
        Ris A = new Ris(x, D, x.mod(2).eq(1), x.sum(2));
        IntLSet E = new IntLSet().ins(2).ins(1);
        IntLVar y = new IntLVar();
        Ris B = new Ris(y, E, y.mod(2).eq(0), y.sum(2));
        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(false, solver.check());
	}

    @Test
    public void testEqInitRisNotInitRisTrue1() {
        IntLSet D = new IntLSet().ins(3);
        IntLVar x = new IntLVar();
        Ris A = new Ris(x, D, x.mod(2).eq(1), x.sum(1));
        IntLSet E = new IntLSet();
        IntLVar y = new IntLVar();
        Ris B = new Ris(y, E, y.mod(2).eq(0), y.sum(2));
        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(true, solver.check());
        assertEquals("{2/_?}", E.toString());
        assertEquals("{3/_?}", D.toString());
        assertEquals(false, solver.nextSolution());
    }
    @Test
    public void testEqInitRisNotInitRisFalse1(){
        IntLSet D = new IntLSet().ins(3);
        IntLVar x = new IntLVar();
        Ris A = new Ris(x, D, x.mod(2).eq(1), x.sum(2));
        IntLSet E = new IntLSet();
        IntLVar y = new IntLVar();
        Ris B = new Ris(y, E, y.mod(2).eq(0), y.sum(2));
        Solver solver = new Solver();
        solver.add(A.eq(B));
        assertEquals(false, solver.check());
    }

    @Test
    public void testEqNotInitRisEmptysetTrue() {
        LSet A = LSet.empty();
        LSet D = new LSet();
        Ris B = new Ris(new LVar(), D, Constraint.truec());

        Solver solver = new Solver();
        solver.add(B.eq(A));
        assertEquals(true, solver.check());
        assertEquals("{ _? : _? | true @ _? } = {}", solver.getConstraint().toString());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqNotInitRisInitSetTrue01(){
        LSet A = new LSet().ins("a").ins("b").ins("c");
        LSet D = new LSet("D");
        Ris B = new Ris(new LVar(), D, Constraint.truec());

        Solver solver = new Solver();
        solver.add(B.eq(A));
        assertEquals(true, solver.check());
        assertEquals("{c,b,a/_?}", D.toString());
        assertEquals("{c,b,a/{ _? : _? | true @ _? }}", A.toString());
        assertEquals("[]", solver.getConstraint().toString());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqNotInitRisInitSetTrue02()  {
        LSet A = LSet.empty().ins("a").ins("b").ins("c");
        LSet D = new LSet("D");
        Ris B = new Ris(new LVar(), D, Constraint.truec());

        Solver solver = new Solver();
        solver.add(B.eq(A));
        assertEquals(true, solver.check());
        assertEquals("{c,b,a/_?}", D.toString());
        assertEquals("{c,b,a}", A.toString());
        assertEquals("{ _? : _? | true @ _? } = {}", solver.getConstraint().toString());

        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqNotInitRisInitSetTrue03() {
        LSet A = LSet.empty().ins(1).ins(2).ins(3);
        LSet D = new LSet("D");
        IntLVar x = new IntLVar();
        Ris B = new Ris(x, D, x.lt(7), x.sub(2));

        Solver solver = new Solver();
        solver.add(B.eq(A));
        assertEquals(true, solver.check());
        assertEquals("{5,4,3/_?}", D.toString());
        ConstraintStringComparator constraintStringComparator = new ConstraintStringComparator(
                "{ _? : _? | _? < 7 @ _?, _? = _? - 2 } = {}",
                solver.getConstraint()
        );
        assertTrue(constraintStringComparator.areEqualInSameOrder());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqNotInitRisInitSetFalse01() {
        LSet A = LSet.empty().ins("a").ins("b").ins("c");
        LSet D = new LSet("D").ins("d");
        LVar x = new LVar("x");
        Ris B = new Ris(x, D, x.neq("b"));

        Solver solver = new Solver();
        solver.add(B.eq(A));
        assertEquals(false, solver.check());
    }

    @Test
    public void testEqNotInitRisInitSetFalse02() {
        LSet A = new LSet().ins("a").ins("b").ins("c");
        LSet D = new LSet("D").ins("d");
        LVar x = new LVar("x");
        Ris B = new Ris(x, D, x.neq("b"));

        Solver solver = new Solver();
        solver.add(B.eq(A));
        assertEquals(false, solver.check());
    }

    @Test
    public void testEqNotInitRisInitSetFalse03() {
        IntLSet A = new IntLSet().ins(1).ins(3).ins(2);
        IntLSet D = new IntLSet("D").ins(0);
        IntLVar x = new IntLVar("x");
        Ris B = new Ris(x, D, x.lt(1), x.sum(2));

        Solver solver = new Solver();
        solver.add(B.eq(A));
        assertEquals(false, solver.check());
    }

    @Test
    public void testEqNotInitRisNotInitSetTrue() {
        LSet A = new LSet("A");
        LSet D = new LSet("D");
        LVar x = new LVar();
        Ris B = new Ris(x, D, x.neq("a"));

        Solver solver = new Solver();
        solver.add(B.eq(A));

        assertEquals(true, solver.check());
        assertEquals("{ _? : _D | _? neq a @ _? }", A.toString());
        assertEquals("[]", solver.getConstraint().toString());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testEqNotInitRisEmptyRisTRUE() {
        Ris A = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        Ris B = new Ris(new LVar(), new LSet(), Constraint.truec());

        Solver solver = new Solver();
        solver.add(B.eq(A));
        assertEquals(true, solver.check());
        assertEquals("{ _? : _? | true @ _? } = {}", solver.getConstraint().toString());
    }

    @Test
    public void testEqNotInitRisInitRisTrue1() {
        IntLSet D = new IntLSet().ins(3);
        IntLVar x = new IntLVar();
        Ris A = new Ris(x, D, x.mod(2).eq(1), x.sum(1));
        IntLSet E = new IntLSet();
        IntLVar y = new IntLVar();
        Ris B = new Ris(y, E, y.mod(2).eq(0), y.sum(2));
        Solver solver = new Solver();
        solver.add(B.eq(A));
        assertEquals(true, solver.check());
        assertEquals("{2/_?}", E.toString());
        assertEquals("{3/_?}", D.toString());
        assertEquals(false, solver.nextSolution());
    }
    @Test
    public void testEqNotInitRisInitRisFalse1() {
        IntLSet D = new IntLSet().ins(3);
        IntLVar x = new IntLVar();
        Ris A = new Ris(x, D, x.mod(2).eq(1), x.sum(2));
        IntLSet E = new IntLSet();
        IntLVar y = new IntLVar();
        Ris B = new Ris(y, E, y.mod(2).eq(0), y.sum(2));
        Solver solver = new Solver();
        solver.add(B.eq(A));
        assertEquals(false, solver.check());
    }


    @Test
    public void testEqSimilDomain() throws Throwable{
        IntLSet C1 = new IntLSet();
        IntLVar x = new IntLVar("x");
        IntLVar y = new IntLVar("y");

        Ris r1 = new Ris(x, C1, x.mod(2).eq(0));
        Ris r2 = new Ris(y, C1.ins(0), y.mod(2).eq(0));

        r1.eq(r2).solve();
    }

    /*
     -------------------------------------------------------------------------------------------------------------
      _____ _   _ ______ ____  _    _         _      _____ _________     __  _______ ______  _____ _______ _____
     /_   _| \ | |  ____/ __ \| |  | |  /\   | |    /_   _/__   __\ \   / / /__   __|  ____|/ ____/__   __/ ____|
       | | |  \| | /__ | |  | | |  | | /  \  | |      | |    | |   \ \_/ /     | |  | /__  | (___    | | | (___
       | | | . ` |  __|| |  | | |  | |/ /\ \ | |      | |    | |    \   /      | |  |  __|  \___ \   | |  \___ \
      _| /_| |\  | /___| /__| | /__| / ____ \| /____ _| /_   | |     | |       | |  | /____ ____) |  | |  ____) |
     /_____/_| \_/______\___\_\\____/_/    \_\______/_____|  /_|     /_|       /_|  /______/_____/   /_| /_____/
     --------------------------------------------------------------------------------------------------------------
     */

    @Test
    public void testNeqEmptySetEmptyRisFalse() throws Failure {
        LSet set = LSet.empty();
        Ris ris = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        Solver solver = new Solver();
        solver.add(set.neq(ris));
        assertEquals(false, solver.check());
    }

    @Test
    public void testNeqEmptySetInitRisTrue1() throws Failure {
        LSet set = LSet.empty();
        IntLVar x = new IntLVar("x");
        IntLSet domain = new IntLSet("D").ins(-3).ins(2);
        Ris ris = new Ris(x, domain, x.mul(x).neq(4));


        Solver solver = new Solver();
        solver.add(set.neq(ris));
        assertEquals(true, solver.check());
    }

    @Test
    public void testNeqEmptySetInitRisTrue2() throws Failure {
        LSet set = LSet.empty();
        IntLVar x = new IntLVar("x");
        IntLSet domain = new IntLSet("D").ins(-2).ins(2);
        Ris ris = new Ris(x, domain, x.mul(x).neq(4));


        Solver solver = new Solver();
        solver.add(set.neq(ris));
        assertEquals(true, solver.check());
    }

    @Test
    public void testNeqEmptySetInitRisTrue3() throws Failure {
        LSet set = LSet.empty();
        IntLVar x = new IntLVar("x");
        IntLSet domain = IntLSet.empty().ins(-3).ins(2);
        Ris ris = new Ris(x, domain, x.mul(x).neq(4));


        Solver solver = new Solver();
        solver.add(set.neq(ris));
        assertEquals(true, solver.check());
    }

    @Test
    public void testNeqEmptySetInitRisFalse1() throws Failure {
        LSet set = LSet.empty();
        IntLVar x = new IntLVar("x");
        IntLSet domain = IntLSet.empty().ins(-2).ins(2);
        Ris ris = new Ris(x, domain, x.mul(x).neq(4));


        Solver solver = new Solver();
        solver.add(set.neq(ris));
        assertEquals(false, solver.check());
    }

    @Test
    public void testNeqEmptySetInitRisFalse2() throws Failure {
        LSet set = LSet.empty();
        IntLVar x = new IntLVar("x");
        IntLSet domain = new IntLSet().ins(-2).ins(2);
        Ris ris = new Ris(x, domain, x.neq(x));


        Solver solver = new Solver();
        solver.add(set.neq(ris));
        assertEquals(false, solver.check());
    }


    @Test
    public void testNeqEmptySetNotInitRisTrue1() throws Failure {
        LSet set = LSet.empty();
        IntLVar x = new IntLVar("x");
        IntLSet domain = new IntLSet("D");
        Ris ris = new Ris(x, domain, x.mul(x).neq(4));


        Solver solver = new Solver();
        solver.add(set.neq(ris));
        assertEquals(true, solver.check());
    }

    @Test
    public void testNeqEmptySetNotInitRisFalse1() throws Failure {
        LSet set = LSet.empty();
        IntLVar x = new IntLVar("x");
        IntLSet domain = new IntLSet();
        Ris ris = new Ris(x, domain, x.neq(x));


        Solver solver = new Solver();
        solver.add(set.neq(ris));
        assertEquals(false, solver.check());
    }

    @Test
    public void testNeqEmptySetNotInitRisFalse2() throws Failure {
        LSet set = LSet.empty();
        IntLVar x = new IntLVar("x");
        IntLSet domain = new IntLSet();
        Ris ris = new Ris(x, domain, Constraint.falsec());


        Solver solver = new Solver();
        solver.add(set.neq(ris));
        assertEquals(false, solver.check());
    }

    @Test
    public void testNeqEmptySetNotInitRisFalse3() throws Failure {
        LSet set = LSet.empty();
        IntLVar x = new IntLVar("x");
        IntLSet domain = new IntLSet();
        Ris ris = new Ris(x, domain, x.eq("1").and(x.eq("a")));


        Solver solver = new Solver();
        solver.add(set.neq(ris));
        assertEquals(false, solver.check());
    }

    @Test
    public void testNeqInitSetEmptyRisFalse(){
        LSet set = new LSet().ins(3).ins(2).ins(1);

        Ris ris = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        Solver solver = new Solver();
        solver.add(set.neq(ris));
        assertEquals(true, solver.check());
    }

    @Test
    public void testNeqInitSetInitRisFalse01(){
        IntLSet set = IntLSet.empty().ins(3).ins(2).ins(1);

        IntLVar x = new IntLVar("x");
        IntLSet domain =IntLSet.empty().ins(0).ins(1).ins(2);
        Ris ris = new Ris(x, domain, Constraint.truec(), x.sum(1));

        Solver solver = new Solver();
        solver.add(set.neq(ris));
        assertEquals(false, solver.check());
    }

    @Test
    public void testNeqInitSetInitRisFalse02(){
        IntLSet set = IntLSet.empty().ins(3).ins(2).ins(1);

        IntLVar x = new IntLVar("x");
        IntLSet domain =new IntLSet().ins(0).ins(1).ins(2);
        Ris ris = new Ris(x, domain, x.ge(0).and(x.le(2)),x.sum(1));

        Solver solver = new Solver();
        solver.add(set.neq(ris));
        assertEquals(false, solver.check());
    }

    @Test
    public void testNeqInitSetInitRisTrue01(){
        IntLSet set = IntLSet.empty().ins(3).ins(2).ins(1);

        IntLVar x = new IntLVar("x");
        IntLSet domain =IntLSet.empty().ins(-1).ins(1).ins(2);
        Ris ris = new Ris(x, domain, Constraint.truec(), x.sum(1));

        Solver solver = new Solver();
        solver.add(set.neq(ris));
        assertEquals(true, solver.check());
    }

    @Test
    public void testNeqInitSetInitRisTrue02(){
        IntLSet set = IntLSet.empty().ins(3).ins(2).ins(1);

        IntLVar x = new IntLVar("x");
        IntLSet domain =new IntLSet().ins(0).ins(1).ins(2);
        Ris ris = new Ris(x, domain, x.ge(0).and(x.le(3)), x.sum(1));

        Solver solver = new Solver();
        solver.add(set.neq(ris));
        assertEquals(true, solver.check());
    }

    @Test
    public void testNeqInitSetNotInitRisTrue01(){
        IntLSet set = IntLSet.empty().ins(3).ins(2).ins(1);

        IntLVar x = new IntLVar("x");
        IntLSet domain = new IntLSet();
        Ris ris = new Ris(x, domain, Constraint.truec(), x.sum(1));

        Solver solver = new Solver();
        solver.add(set.neq(ris));
        assertEquals(true, solver.check());
    }

    @Test
    public void testNeqInitSetNotInitRisTrue02(){
        IntLSet set = IntLSet.empty().ins(3).ins(2).ins(1);

        IntLVar x = new IntLVar("x");
        IntLSet domain = new IntLSet();
        Ris ris = new Ris(x, domain, Constraint.truec());

        Solver solver = new Solver();
        solver.add(set.neq(ris));
        assertEquals(true, solver.check());
    }

    @Test
    public void testNeqInitSetNotInitRisTrue03(){
        IntLSet set = IntLSet.empty().ins(3).ins(2).ins(1);

        IntLVar x = new IntLVar("x");
        IntLSet domain =new IntLSet();
        Ris ris = new Ris(x, domain, x.ge(0).and(x.le(3)), x.sum(1));

        Solver solver = new Solver();
        solver.add(set.neq(ris));
        assertEquals(true, solver.check());
    }

    @Test
    public void testNeqNotInitSetEmptyRisTrue(){
        LSet set = new LSet();
        Ris ris = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        assertEquals(true, set.neq(ris).check());
    }

    @Test
    public void testNeqNotInitSetInitRisTrue01(){
        LSet set = new LSet();

        LVar x = new LVar();
        LSet domain = new LSet().ins("a").ins("b").ins("c");
        Ris ris = new Ris(x, domain, x.neq("d").and(x.neq("e")));

        assertEquals(true, set.neq(ris).check());
    }

    @Test
    public void testNeqNotInitSetInitRisTrue02(){
        IntLSet set = new IntLSet();

        IntLVar x = new IntLVar();
        IntLSet domain = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris = new Ris(x, domain, x.lt(3), x.sum(1));

        assertEquals(true, set.neq(ris).check());
    }

    @Test
    public void testNeqNotInitSetInitRisTrue03(){
        LSet set = new LSet();

        LVar x = new LVar();
        LSet domain = set.ins("a").ins("b").ins("c");
        Ris ris = new Ris(x, domain, Constraint.truec());

        Solver solver = new Solver();
        solver.add(set.neq(ris));

        assertEquals(true, set.neq(ris).check());
    }

    @Test
    public void testNeqNotInitSetInitRisFalse01(){
        LSet set = new LSet();

        LVar x = new LVar();
        LSet domain = set.ins("a").ins("b").ins("c");
        Ris ris = new Ris(x, domain, Constraint.truec());

        Solver solver = new Solver();
        solver.add(set.neq(ris));
        solver.add(set.contains("a"));
        solver.add(set.contains("b"));
        solver.add(set.contains("c"));
        assertEquals(false, solver.check());
    }

    @Test
    public void testNeqNotInitSetInitRisFalse02(){
        LSet set = new LSet();

        LVar x = new LVar();
        LSet domain = set.ins("a").ins("b").ins("c");
        Ris ris = new Ris(x, domain, Constraint.truec());

        Solver solver = new Solver();
        solver.add(set.contains("a"));
        solver.add(set.contains("b"));
        solver.add(set.contains("c"));
        solver.add(set.neq(ris));

        assertEquals(false, solver.check());
    }

    @Test
    public void testNeqNotInitSetNotInitRisTrue01(){
        LSet set = new LSet();

        LVar x = new LVar();
        LSet domain = new LSet();
        Ris ris = new Ris(x,domain, Constraint.truec());

        assertEquals(true, set.neq(ris).check());
    }

    @Test
    public void testNeqNotInitSetNotInitRisTrue02(){
        LSet set = new LSet();

        LVar x = new LVar();
        LSet domain = new LSet();
        Ris ris = new Ris(x,domain, x.in(set));

        assertEquals(true, set.neq(ris).check());
    }

    @Test
    public void testNeqNotInitSetNotInitRisTrue03(){
        LSet set = new LSet();

        LVar x = new LVar();

        Ris ris = new Ris(x,set, x.neq(1));

        assertEquals(true, set.neq(ris).check());
    }

    @Test
    public void testNeqNotInitSetNotInitRisFalse01(){
        LSet set = new LSet();

        LVar x = new LVar();

        Ris ris = new Ris(x,set, Constraint.truec());

        assertEquals(false, set.neq(ris).check());
    }

    @Test
    public void testNeqNotInitSetNotInitRisFalse02(){
        LSet set = new LSet();

        LVar x = new LVar();

        Ris ris = new Ris(x,set, x.neq(1).or(x.neq(2)));

        assertEquals(false, set.neq(ris).check());
    }

    @Test
    public void testNeqEmptyRisEmptySetFalse() throws Failure {
        LSet set = LSet.empty();
        Ris ris = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        Solver solver = new Solver();
        solver.add(ris.neq(set));
        assertEquals(false, solver.check());
    }

    @Test
    public void testNeqInitRisEmptySetTrue1() throws Failure {
        LSet set = LSet.empty();
        IntLVar x = new IntLVar("x");
        IntLSet domain = new IntLSet("D").ins(-3).ins(2);
        Ris ris = new Ris(x, domain, x.mul(x).neq(4));


        Solver solver = new Solver();
        solver.add(ris.neq(set));
        assertEquals(true, solver.check());
    }

    @Test
    public void testNeqInitRisEmptySetTrue2() throws Failure {
        LSet set = LSet.empty();
        IntLVar x = new IntLVar("x");
        IntLSet domain = new IntLSet("D").ins(-2).ins(2);
        Ris ris = new Ris(x, domain, x.mul(x).neq(4));


        Solver solver = new Solver();
        solver.add(ris.neq(set));
        assertEquals(true, solver.check());
    }

    @Test
    public void testNeqInitRisEmptySetTrue3() throws Failure {
        LSet set = LSet.empty();
        IntLVar x = new IntLVar("x");
        IntLSet domain = IntLSet.empty().ins(-3).ins(2);
        Ris ris = new Ris(x, domain, x.mul(x).neq(4));


        Solver solver = new Solver();
        solver.add(ris.neq(set));
        assertEquals(true, solver.check());
    }

    @Test
    public void testNeqInitRisEmptySetFalse1() throws Failure {
        LSet set = LSet.empty();
        IntLVar x = new IntLVar("x");
        IntLSet domain = IntLSet.empty().ins(-2).ins(2);
        Ris ris = new Ris(x, domain, x.mul(x).neq(4));


        Solver solver = new Solver();
        solver.add(ris.neq(set));
        assertEquals(false, solver.check());
    }

    @Test
    public void testNeqInitRisEmptySetFalse2() throws Failure {
        LSet set = LSet.empty();
        IntLVar x = new IntLVar("x");
        IntLSet domain = new IntLSet().ins(-2).ins(2);
        Ris ris = new Ris(x, domain, x.neq(x));


        Solver solver = new Solver();
        solver.add(ris.neq(set));
        assertEquals(false, solver.check());
    }

    @Test
    public void testNeqNotInitRisEmptySetFalse1() throws Failure {
        LSet set = LSet.empty();
        IntLVar x = new IntLVar("x");
        IntLSet domain = new IntLSet();
        Ris ris = new Ris(x, domain, x.neq(x));


        Solver solver = new Solver();
        solver.add(ris.neq(set));
        assertEquals(false, solver.check());
    }

    @Test
    public void testNeqNotInitRisEmptySetFalse2() throws Failure {
        LSet set = LSet.empty();
        IntLVar x = new IntLVar("x");
        IntLSet domain = new IntLSet();
        Ris ris = new Ris(x, domain, Constraint.falsec());


        Solver solver = new Solver();
        solver.add(ris.neq(set));
        assertEquals(false, solver.check());
    }

    @Test
    public void testNeqNotInitRisEmptySetFalse3() throws Failure {
        LSet set = LSet.empty();
        IntLVar x = new IntLVar("x");
        IntLSet domain = new IntLSet();
        Ris ris = new Ris(x, domain, x.eq("1").and(x.eq("a")));


        Solver solver = new Solver();
        solver.add(ris.neq(set));
        assertEquals(false, solver.check());
    }

    @Test
    public void testNeqEmptyRisInitSetFalse(){
        LSet set = new LSet().ins(3).ins(2).ins(1);

        Ris ris = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        Solver solver = new Solver();
        solver.add(ris.neq(set));
        assertEquals(true, solver.check());
    }

    @Test
    public void testNeqInitRisInitSetFalse01(){
        IntLSet set = IntLSet.empty().ins(3).ins(2).ins(1);

        IntLVar x = new IntLVar("x");
        IntLSet domain =IntLSet.empty().ins(0).ins(1).ins(2);
        Ris ris = new Ris(x, domain, Constraint.truec(), x.sum(1));

        Solver solver = new Solver();
        solver.add(ris.neq(set));
        assertEquals(false, solver.check());
    }

    @Test
    public void testNeqInitRisInitSetFalse02(){
        IntLSet set = IntLSet.empty().ins(3).ins(2).ins(1);

        IntLVar x = new IntLVar("x");
        IntLSet domain =new IntLSet().ins(0).ins(1).ins(2);
        Ris ris = new Ris(x, domain, x.ge(0).and(x.le(2)), x.sum(1));

        Solver solver = new Solver();
        solver.add(ris.neq(set));
        assertEquals(false, solver.check());
    }

    @Test
    public void testNeqInitRisInitSetTrue01(){
        IntLSet set = IntLSet.empty().ins(3).ins(2).ins(1);

        IntLVar x = new IntLVar("x");
        IntLSet domain =IntLSet.empty().ins(-1).ins(1).ins(2);
        Ris ris = new Ris(x, domain, Constraint.truec(), x.sum(1));

        Solver solver = new Solver();
        solver.add(ris.neq(set));
        assertEquals(true, solver.check());
    }

    @Test
    public void testNeqInitRisInitSetTrue02(){
        IntLSet set = IntLSet.empty().ins(3).ins(2).ins(1);

        IntLVar x = new IntLVar("x");
        IntLSet domain =new IntLSet().ins(0).ins(1).ins(2);
        Ris ris = new Ris(x, domain, x.ge(0).and(x.le(3)), x.sum(1));

        Solver solver = new Solver();
        solver.add(ris.neq(set));
        assertEquals(true, solver.check());
    }

    @Test
    public void testNeqNotInitRisInitSetTrue01(){
        IntLSet set = IntLSet.empty().ins(3).ins(2).ins(1);

        IntLVar x = new IntLVar("x");
        IntLSet domain = new IntLSet();
        Ris ris = new Ris(x, domain, Constraint.truec(), x.sum(1));

        Solver solver = new Solver();
        solver.add(ris.neq(set));
        assertEquals(true, solver.check());
    }

    @Test
    public void testNeqNotInitRisInitSetTrue02(){
        IntLSet set = IntLSet.empty().ins(3).ins(2).ins(1);

        IntLVar x = new IntLVar("x");
        IntLSet domain = new IntLSet();
        Ris ris = new Ris(x, domain, Constraint.truec());

        Solver solver = new Solver();
        solver.add(ris.neq(set));
        assertEquals(true, solver.check());
    }

    @Test
    public void testNeqNotInitRisInitSetTrue03(){
        IntLSet set = IntLSet.empty().ins(3).ins(2).ins(1);

        IntLVar x = new IntLVar("x");
        IntLSet domain =new IntLSet();
        Ris ris = new Ris(x, domain, x.ge(0).and(x.le(3)), x.sum(1));

        Solver solver = new Solver();
        solver.add(ris.neq(set));
        assertEquals(true, solver.check());
    }

    @Test
    public void testNeqEmptyRisNotInitSetTrue(){
        LSet set = new LSet();
        Ris ris = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        assertEquals(true, ris.neq(set).check());
    }

    @Test
    public void testNeqInitRisNotInitSetTrue01(){
        LSet set = new LSet();

        LVar x = new LVar();
        LSet domain = new LSet().ins("a").ins("b").ins("c");
        Ris ris = new Ris(x, domain, x.neq("d").and(x.neq("e")));

        assertEquals(true, ris.neq(set).check());
    }

    @Test
    public void testNeqInitRisNotInitSetTrue02(){
        IntLSet set = new IntLSet();

        IntLVar x = new IntLVar();
        IntLSet domain = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris = new Ris(x, domain, x.lt(3), x.sum(1));

        assertEquals(true, ris.neq(set).check());
    }

    @Test
    public void testNeqInitRisNotInitSetTrue03(){
        LSet set = new LSet();

        LVar x = new LVar();
        LSet domain = set.ins("a").ins("b").ins("c");
        Ris ris = new Ris(x, domain, Constraint.truec());

        Solver solver = new Solver();
        solver.add(set.neq(ris));

        assertEquals(true, ris.neq(set).check());
    }

    @Test
    public void testNeqInitRisNotInitSetFalse01(){
        LSet set = new LSet();

        LVar x = new LVar();
        LSet domain = set.ins("a").ins("b").ins("c");
        Ris ris = new Ris(x, domain, Constraint.truec());

        Solver solver = new Solver();
        solver.add(ris.neq(set));
        solver.add(set.contains("a"));
        solver.add(set.contains("b"));
        solver.add(set.contains("c"));
        assertEquals(false, solver.check());
    }

    @Test
    public void testNeqInitRisNotInitSetFalse02(){
        LSet set = new LSet();

        LVar x = new LVar();
        LSet domain = set.ins("a").ins("b").ins("c");
        Ris ris = new Ris(x, domain, Constraint.truec());

        Solver solver = new Solver();
        solver.add(set.contains("a"));
        solver.add(set.contains("b"));
        solver.add(set.contains("c"));
        solver.add(ris.neq(set));

        assertEquals(false, solver.check());
    }

    @Test
    public void testNeqNotInitRisNotInitSetTrue01(){
        LSet set = new LSet();

        LVar x = new LVar();
        LSet domain = new LSet();
        Ris ris = new Ris(x,domain, Constraint.truec());

        assertEquals(true, ris.neq(set).check());
    }

    @Test
    public void testNeqNotInitRisNotInitSetTrue02(){
        LSet set = new LSet();

        LVar x = new LVar();
        LSet domain = new LSet();
        Ris ris = new Ris(x,domain, x.in(set));

        assertEquals(true, ris.neq(set).check());
    }

    @Test
    public void testNeqNotInitRisNotInitSetTrue03(){
        LSet set = new LSet();

        LVar x = new LVar();

        Ris ris = new Ris(x,set, x.neq(1));

        assertEquals(true, ris.neq(set).check());
    }

    @Test
    public void testNeqNotInitRisNotInitSetFalse01(){
        LSet set = new LSet();

        LVar x = new LVar();

        Ris ris = new Ris(x,set, Constraint.truec());

        assertEquals(false, ris.neq(set).check());
    }

    @Test
    public void testNeqNotInitRisNotInitSetFalse02(){
        LSet set = new LSet();

        LVar x = new LVar();

        Ris ris = new Ris(x,set, x.neq(1).or(x.neq(2)));

        assertEquals(false, ris.neq(set).check());
    }


    @Test
    public void testNeqEmptyRisEmptyRisFalse(){
        Ris ris1 = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        Ris ris2 = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        assertEquals(false, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqEmptyRisInitRisFalse1(){
        Ris ris1 = new Ris(new IntLVar(), LSet.empty(), Constraint.truec());

        IntLVar x = new IntLVar();
        IntLSet domain = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(x, domain, x.gt(3));

        assertEquals(false, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqEmptyRisInitRisFalse2(){
        Ris ris1 = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        IntLVar x = new IntLVar();
        IntLSet domain = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(x, domain, Constraint.falsec());

        assertEquals(false, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqEmptyRisInitRisFalse3(){
        Ris ris1 = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        IntLVar x = new IntLVar();
        IntLSet domain = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(x, domain, Constraint.falsec());

        assertEquals(false, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqEmptyRisInitRisTrue1(){
        Ris ris1 = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        IntLVar x = new IntLVar();
        IntLSet domain = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(x, domain, x.gt(2));

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqEmptyRisInitRisTrue2(){
        Ris ris1 = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        IntLVar x = new IntLVar();
        IntLSet domain = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(x, domain, Constraint.truec());

        assertEquals(true, ris1.neq(ris2).check());
    }


    @Test
    public void testNeqEmptyRisInitRisTrue3(){
        Ris ris1 = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        IntLVar x = new IntLVar();
        IntLSet domain = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(x, domain, Constraint.truec());

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqEmptyRisInitRisTrue4(){
        Ris ris1 = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        IntLVar x = new IntLVar();
        IntLSet domain = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(x, domain, x.gt(2));

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqEmptyRisInitRisTrue5(){
        Ris ris1 = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        IntLVar x = new IntLVar();
        IntLSet domain = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(x, domain, x.gt(4));

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqEmptyRisNotInitRisFalse1(){
        Ris ris1 = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        IntLVar x = new IntLVar();
        IntLSet domain = new IntLSet();
        Ris ris2 = new Ris(x, domain, x.gt(3).and(x.lt(3)));

        assertEquals(false, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqEmptyRisNotInitRisFalse2(){
        Ris ris1 = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        IntLVar x = new IntLVar();
        IntLSet domain = new IntLSet();
        Ris ris2 = new Ris(x, domain, Constraint.falsec());

        assertEquals(false, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqEmptyRisNotInitRisFalse3(){
        Ris ris1 = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        IntLVar x = new IntLVar();
        IntLSet domain = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(x, domain, Constraint.falsec());

        assertEquals(false, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqEmptyRisNotInitRisTrue1(){
        Ris ris1 = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        IntLVar x = new IntLVar();
        IntLSet domain = new IntLSet();
        Ris ris2 = new Ris(x, domain, x.gt(2));

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqEmptyRisNotInitRisTrue2(){
        Ris ris1 = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        IntLVar x = new IntLVar();
        IntLSet domain = new IntLSet();
        Ris ris2 = new Ris(x, domain, Constraint.truec());

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisEmptyRisFalse1(){
        Ris ris1 = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        IntLVar x = new IntLVar();
        IntLSet domain = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(x, domain, x.gt(3));

        assertEquals(false, ris2.neq(ris1).check());
    }

    @Test
    public void testNeqInitRisEmptyRisFalse2(){
        Ris ris1 = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        IntLVar x = new IntLVar();
        IntLSet domain = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(x, domain, Constraint.falsec());

        assertEquals(false, ris2.neq(ris1).check());
    }

    @Test
    public void testNeqInitRisEmptyRisFalse3(){
        Ris ris1 = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        IntLVar x = new IntLVar();
        IntLSet domain = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(x, domain, Constraint.falsec());

        assertEquals(false, ris2.neq(ris1).check());
    }

    @Test
    public void testNeqInitRisEmptyRisTrue1(){
        Ris ris1 = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        IntLVar x = new IntLVar();
        IntLSet domain = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(x, domain, x.gt(2));

        assertEquals(true, ris2.neq(ris1).check());
    }

    @Test
    public void testNeqInitRisEmptyRisTrue2(){
        Ris ris1 = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        IntLVar x = new IntLVar();
        IntLSet domain = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(x, domain, Constraint.truec());

        assertEquals(true, ris2.neq(ris1).check());
    }


    @Test
    public void testNeqInitRisEmptyRisTrue3(){
        Ris ris1 = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        IntLVar x = new IntLVar();
        IntLSet domain = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(x, domain, Constraint.truec());

        assertEquals(true, ris2.neq(ris1).check());
    }

    @Test
    public void testNeqInitRisEmptyRisTrue4(){
        Ris ris1 = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        IntLVar x = new IntLVar();
        IntLSet domain = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(x, domain, x.gt(2));

        assertEquals(true, ris2.neq(ris1).check());
    }

    @Test
    public void testNeqInitRisEmptyRisTrue5(){
        Ris ris1 = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        IntLVar x = new IntLVar();
        IntLSet domain = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(x, domain, x.gt(4));

        assertEquals(true, ris2.neq(ris1).check());
    }

    @Test
    public void testNeqInitRisInitRisTrue01(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, x.mod(2).eq(1));

        IntLVar y = new IntLVar();
        IntLSet domain2 = IntLSet.empty().ins(3).ins(4).ins(5);
        Ris ris2 = new Ris(x, domain2, x.mod(2).eq(1));

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisTrue02(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, x.mod(2).eq(0));

        IntLVar y = new IntLVar();
        IntLSet domain2 = IntLSet.empty().ins(3).ins(4).ins(5);
        Ris ris2 = new Ris(x, domain2, x.mod(2).eq(1));

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisTrue03(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, x.mod(2).eq(1));

        IntLVar y = new IntLVar();
        IntLSet domain2 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(x, domain2, x.mod(2).eq(0));

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisTrue04(){
        IntLVar x = new IntLVar();
        IntLSet domain = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain, x.mod(2).eq(1));

        IntLVar y = new IntLVar();
        Ris ris2 = new Ris(x, domain, x.mod(2).eq(0));

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisTrue05(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, Constraint.truec());

        IntLVar y = new IntLVar();
        IntLSet domain2 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(x, domain2, x.mod(2).eq(0));

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisTrue06(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, Constraint.truec());

        IntLVar y = new IntLVar();
        IntLSet domain2 = IntLSet.empty().ins(2).ins(3).ins(4);
        Ris ris2 = new Ris(x, domain2, Constraint.truec());

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisTrue07(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, Constraint.falsec());

        IntLVar y = new IntLVar();
        IntLSet domain2 = IntLSet.empty().ins(2).ins(3).ins(4);
        Ris ris2 = new Ris(x, domain2, Constraint.truec());

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisTrue08(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, Constraint.truec());

        IntLVar y = new IntLVar();
        IntLSet domain2 = IntLSet.empty().ins(2).ins(3).ins(4);
        Ris ris2 = new Ris(x, domain2, Constraint.falsec());

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisTrue09(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, x.mod(2).eq(1));

        IntLVar y = new IntLVar();
        IntLSet domain2 = IntLSet.empty().ins(3).ins(4).ins(5);
        Ris ris2 = new Ris(x, domain2, x.mod(2).eq(1));

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisTrue10(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, x.mod(2).eq(1));

        IntLVar y = new IntLVar();
        IntLSet domain2 = new IntLSet().ins(3).ins(4).ins(5);
        Ris ris2 = new Ris(x, domain2, x.mod(2).eq(1));

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisTrue11(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, x.mod(2).eq(1));

        IntLVar y = new IntLVar();
        IntLSet domain2 = new IntLSet().ins(3).ins(4).ins(5);
        Ris ris2 = new Ris(x, domain2, x.mod(2).eq(1));

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisTrue12(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, x.mod(2).eq(0));

        IntLVar y = new IntLVar();
        IntLSet domain2 = IntLSet.empty().ins(3).ins(4).ins(5);
        Ris ris2 = new Ris(x, domain2, x.mod(2).eq(1));

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisTrue13(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, x.mod(2).eq(0));

        IntLVar y = new IntLVar();
        IntLSet domain2 = new IntLSet().ins(3).ins(4).ins(5);
        Ris ris2 = new Ris(x, domain2, x.mod(2).eq(1));

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisTrue14(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, x.mod(2).eq(0));

        IntLVar y = new IntLVar();
        IntLSet domain2 = new IntLSet().ins(3).ins(4).ins(5);
        Ris ris2 = new Ris(x, domain2, x.mod(2).eq(1));

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisTrue15(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, x.mod(2).eq(1));

        IntLVar y = new IntLVar();
        IntLSet domain2 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(x, domain2, x.mod(2).eq(0));

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisTrue16(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, x.mod(2).eq(1));

        IntLVar y = new IntLVar();
        IntLSet domain2 = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(x, domain2, x.mod(2).eq(0));

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisTrue17(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, x.mod(2).eq(1));

        IntLVar y = new IntLVar();
        IntLSet domain2 = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(x, domain2, x.mod(2).eq(0));

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisTrue18(){
        IntLVar x = new IntLVar();
        IntLSet domain = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain, x.mod(2).eq(1));

        IntLVar y = new IntLVar();
        Ris ris2 = new Ris(x, domain, x.mod(2).eq(0));

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisTrue19(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, Constraint.truec());

        IntLVar y = new IntLVar();
        IntLSet domain2 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(x, domain2, x.mod(2).eq(0));

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisTrue20(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, Constraint.truec());

        IntLVar y = new IntLVar();
        IntLSet domain2 = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(x, domain2, x.mod(2).eq(0));

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisTrue21(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, Constraint.truec());

        IntLVar y = new IntLVar();
        IntLSet domain2 = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(x, domain2, x.mod(2).eq(0));

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisTrue22(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, Constraint.truec());

        IntLVar y = new IntLVar();
        IntLSet domain2 = IntLSet.empty().ins(2).ins(3).ins(4);
        Ris ris2 = new Ris(x, domain2, Constraint.truec());

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisTrue23(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, Constraint.truec());

        IntLVar y = new IntLVar();
        IntLSet domain2 = new IntLSet().ins(2).ins(3).ins(4);
        Ris ris2 = new Ris(x, domain2, Constraint.truec());

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisTrue24(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, Constraint.truec());

        IntLVar y = new IntLVar();
        IntLSet domain2 = new IntLSet().ins(2).ins(3).ins(4);
        Ris ris2 = new Ris(x, domain2, Constraint.truec());

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisTrue25(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, Constraint.falsec());

        IntLVar y = new IntLVar();
        IntLSet domain2 = IntLSet.empty().ins(2).ins(3).ins(4);
        Ris ris2 = new Ris(x, domain2, Constraint.truec());

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisTrue26(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, Constraint.falsec());

        IntLVar y = new IntLVar();
        IntLSet domain2 = new IntLSet().ins(2).ins(3).ins(4);
        Ris ris2 = new Ris(x, domain2, Constraint.truec());

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisTrue27(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, Constraint.falsec());

        IntLVar y = new IntLVar();
        IntLSet domain2 = new IntLSet().ins(2).ins(3).ins(4);
        Ris ris2 = new Ris(x, domain2, Constraint.truec());

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisTrue28(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, Constraint.truec());

        IntLVar y = new IntLVar();
        IntLSet domain2 = IntLSet.empty().ins(2).ins(3).ins(4);
        Ris ris2 = new Ris(x, domain2, Constraint.falsec());

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisTrue29(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, Constraint.truec());

        IntLVar y = new IntLVar();
        IntLSet domain2 = new IntLSet().ins(2).ins(3).ins(4);
        Ris ris2 = new Ris(x, domain2, Constraint.falsec());

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisTrue30(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, Constraint.truec());

        IntLVar y = new IntLVar();
        IntLSet domain2 = new IntLSet().ins(2).ins(3).ins(4);
        Ris ris2 = new Ris(x, domain2, Constraint.falsec());

        assertEquals(true, ris1.neq(ris2).check());
    }


    @Test
    public void testNeqInitRisInitRisFalse01(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, x.mod(2).eq(1));

        IntLVar y = new IntLVar();
        IntLSet domain2 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(x, domain2, x.mod(2).eq(1));

        assertEquals(false, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisFalse02(){
        IntLVar x = new IntLVar();
        IntLSet domain = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain, x.mod(2).eq(1));

        IntLVar y = new IntLVar();
        Ris ris2 = new Ris(x, domain, x.mod(2).eq(1));

        assertEquals(false, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisFalse03(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, Constraint.truec());

        IntLVar y = new IntLVar();
        IntLSet domain2 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(y, domain2, y.mod(2).eq(0).or(y.mod(2).eq(1)));

        assertEquals(false, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisFalse04(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, Constraint.truec());

        IntLVar y = new IntLVar();
        IntLSet domain2 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(x, domain2, Constraint.truec());

        assertEquals(false, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisFalse05(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, Constraint.truec());

        IntLVar y = new IntLVar();
        IntLSet domain2 = IntLSet.empty().ins(1).ins(3).ins(2);
        Ris ris2 = new Ris(x, domain2, Constraint.truec());

        assertEquals(false, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisFalse06(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, Constraint.truec());

        IntLVar y = new IntLVar();
        IntLSet domain2 = IntLSet.empty().ins(2).ins(1).ins(3);
        Ris ris2 = new Ris(x, domain2, Constraint.truec());

        assertEquals(false, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisFalse07(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, Constraint.truec());

        IntLVar y = new IntLVar();
        IntLSet domain2 = IntLSet.empty().ins(2).ins(3).ins(1);
        Ris ris2 = new Ris(x, domain2, Constraint.truec());

        assertEquals(false, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisFalse08(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, Constraint.truec());

        IntLVar y = new IntLVar();
        IntLSet domain2 = IntLSet.empty().ins(3).ins(1).ins(2);
        Ris ris2 = new Ris(x, domain2, Constraint.truec());

        assertEquals(false, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisFalse09(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, Constraint.truec());

        IntLVar y = new IntLVar();
        IntLSet domain2 = IntLSet.empty().ins(3).ins(2).ins(1);
        Ris ris2 = new Ris(x, domain2, Constraint.truec());

        assertEquals(false, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisFalse10(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, Constraint.falsec());

        IntLVar y = new IntLVar();
        IntLSet domain2 = IntLSet.empty().ins(2).ins(3).ins(4);
        Ris ris2 = new Ris(y, domain2, Constraint.falsec());

        assertEquals(false, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisFalse11(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, Constraint.falsec());

        IntLVar y = new IntLVar();
        IntLSet domain2 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(y, domain2, Constraint.falsec());

        assertEquals(false, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisFalse12(){
        IntLVar x = new IntLVar();
        IntLSet domain = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain, Constraint.falsec());

        IntLVar y = new IntLVar();
        Ris ris2 = new Ris(y, domain, Constraint.falsec());

        assertEquals(false, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisFalse13(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, x.ge(1).and(x.le(3)));

        IntLVar y = new IntLVar();
        IntLSet domain2 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(y, domain2, Constraint.truec());

        assertEquals(false, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisInitRisFalse14(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, x.ge(1).and(x.le(3)));

        IntLVar y = new IntLVar();
        IntLSet domain2 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(y, domain2, Constraint.truec());

        assertEquals(false, ris2.neq(ris1).check());
    }

    @Test
    public void testNeqInitRisInitRisFalse15(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, x.ge(1).and(x.le(3)));

        IntLVar y = new IntLVar();
        IntLSet domain2 = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(y, domain2, y.ge(1).and(y.le(3)));

        assertEquals(false, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisNotInitRisTrue01(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, Constraint.truec());

        IntLVar y = new IntLVar();
        IntLSet domain2 = new IntLSet();
        Ris ris2 = new Ris(y, domain2, Constraint.truec());

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisNotInitRisTrue02(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, x.gt(2));

        IntLVar y = new IntLVar();
        IntLSet domain2 = new IntLSet();
        Ris ris2 = new Ris(y, domain2, x.gt(1));

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqInitRisNotInitRisFalse01(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, IntLSet.empty(), Constraint.truec());

        IntLVar y = new IntLVar();
        IntLSet domain2 = new IntLSet();
        Ris ris2 = new Ris(y, domain2, Constraint.falsec());

        assertEquals(false, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqNotInitRisEmptySetTrue1() throws Failure {
        LSet set = LSet.empty();
        IntLVar x = new IntLVar("x");
        IntLSet domain = new IntLSet("D");
        Ris ris = new Ris(x, domain, x.mul(x).neq(4));


        Solver solver = new Solver();
        solver.add(ris.neq(set));
        assertEquals(true, solver.check());
    }

    @Test
    public void testNeqNotInitRisEmptyRisFalse1(){
        Ris ris1 = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        IntLVar x = new IntLVar();
        IntLSet domain = new IntLSet();
        Ris ris2 = new Ris(x, domain, x.gt(3).and(x.lt(3)));

        assertEquals(false, ris2.neq(ris1).check());
    }

    @Test
    public void testNotInitRisNeqEmptyRisFalse2(){
        Ris ris1 = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        IntLVar x = new IntLVar();
        IntLSet domain = new IntLSet();
        Ris ris2 = new Ris(x, domain, Constraint.falsec());

        assertEquals(false, ris2.neq(ris1).check());
    }

    @Test
    public void testNeqNotInitRisEmptyRisFalse3(){
        Ris ris1 = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        IntLVar x = new IntLVar();
        IntLSet domain = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris2 = new Ris(x, domain, Constraint.falsec());

        assertEquals(false, ris2.neq(ris1).check());
    }

    @Test
    public void testNeqNotInitRisEmptyRisTrue1(){
        Ris ris1 = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        IntLVar x = new IntLVar();
        IntLSet domain = new IntLSet();
        Ris ris2 = new Ris(x, domain, x.gt(2));

        assertEquals(true, ris2.neq(ris1).check());
    }

    @Test
    public void testNeqNotInitRisEmptyRisTrue2(){
        Ris ris1 = new Ris(new LVar(), LSet.empty(), Constraint.truec());

        IntLVar x = new IntLVar();
        IntLSet domain = new IntLSet();
        Ris ris2 = new Ris(x, domain, Constraint.truec());

        assertEquals(true, ris2.neq(ris1).check());
    }

    @Test
    public void testNeqNotInitRisInitRisTrue01(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, Constraint.truec());

        IntLVar y = new IntLVar();
        IntLSet domain2 = new IntLSet();
        Ris ris2 = new Ris(y, domain2, Constraint.truec());

        assertEquals(true, ris2.neq(ris1).check());
    }

    @Test
    public void testNeqNotInitRisInitRisTrue02(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = IntLSet.empty().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, x.gt(2));

        IntLVar y = new IntLVar();
        IntLSet domain2 = new IntLSet();
        Ris ris2 = new Ris(y, domain2, x.gt(1));

        assertEquals(true, ris2.neq(ris1).check());
    }

    @Test
    public void testNeqNotInitRisInitRisTrue03(){
        IntLVar x = new IntLVar();
        IntLSet domain1 = new IntLSet().ins(1).ins(2).ins(3);
        Ris ris1 = new Ris(x, domain1, x.gt(2));

        IntLVar y = new IntLVar();
        IntLSet domain2 = new IntLSet();
        Ris ris2 = new Ris(y, domain2, x.gt(1));

        assertEquals(true, ris2.neq(ris1).check());
    }

    @Test
    public void testNeqNotInitRisInitRisFalse01(){
        IntLVar x = new IntLVar();
        Ris ris1 = new Ris(x, IntLSet.empty(), Constraint.truec());

        IntLVar y = new IntLVar();
        IntLSet domain2 = new IntLSet();
        Ris ris2 = new Ris(y, domain2, Constraint.falsec());

        assertEquals(false, ris2.neq(ris1).check());
    }

    @Test
    public void testNeqNotInitRisNotInitRisFalse01(){
        IntLVar x = new IntLVar();
        Ris ris1 = new Ris(x, new IntLSet(), Constraint.falsec());

        IntLVar y = new IntLVar();
        Ris ris2 = new Ris(y, new IntLSet(), Constraint.falsec());

        assertEquals(false, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqNotInitRisNotInitRisFalse02(){
        IntLVar x = new IntLVar();
        IntLSet domain = new IntLSet();
        Ris ris1 = new Ris(x, domain, x.eq(1).or(x.eq(2)).or(x.eq(3)));

        IntLVar y = new IntLVar();
        Ris ris2 = new Ris(y, domain, y.eq(1).or(y.eq(3)).or(y.eq(2)));

        assertEquals(false, ris1.neq(ris2).check());
    }



    @Test
    public void testNeqNotInitRisNotInitRisTrue01(){
        IntLVar x = new IntLVar();
        Ris ris1 = new Ris(x, new IntLSet(), Constraint.truec());

        IntLVar y = new IntLVar();
        Ris ris2 = new Ris(y, new IntLSet(), Constraint.falsec());

        assertEquals(true, ris1.neq(ris2).check());
    }

    @Test
    public void testNeqNotInitRisNotInitRisTrue02(){
        IntLVar x = new IntLVar();
        IntLSet domain = new IntLSet();
        Ris ris1 = new Ris(x, domain, x.eq(1).or(x.eq(4)).or(x.eq(3)));

        IntLVar y = new IntLVar();
        Ris ris2 = new Ris(y, domain, y.eq(1).or(y.eq(3)).or(y.eq(2)));

        assertEquals(true, ris1.neq(ris2).check());
    }

    /*--------------------------------------------------------------------------------
      _____ _   _   _______ ______  _____ _______ _____
     /_   _| \ | | /__   __|  ____|/ ____/__   __/ ____|
       | | |  \| |    | |  | /__  | (___    | | | (___
       | | | . ` |    | |  |  __|  \___ \   | |  \___ \
      _| /_| |\  |    | |  | /____ ____) |  | |  ____) |
     /_____/_| \_|    /_|  /______/_____/   /_| /_____/

    --------------------------------------------------------------------------------*/

    @Test
    public void testInObjectEmptyRisFalse1(){
        Ris ris = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        assertEquals(false, ris.contains("something").check());
    }

    @Test
    public void testInNotInitLVarEmptyRisFalse1(){
        Ris ris = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        LVar lVar = new LVar();
        assertEquals(false, lVar.in(ris).check());
    }

    @Test
    public void testInInitLVarEmptyRisFalse1(){
        Ris ris = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        LVar lVar = new LVar("something");
        assertEquals(false, lVar.in(ris).check());
    }

    @Test
    public void testInObjectInitRisTrue1(){
        Object object = "something";

        LVar x = new LVar();
        LSet domain = LSet.empty().ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, x.neq("somethingDifferent"));

        assertEquals(true, ris.contains(object).check());
    }

    @Test
    public void testInObjectInitRisTrue2(){
        Object object = "something";

        LVar x = new LVar();
        LSet domain = new LSet().ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, x.neq("somethingDifferent"));

        assertEquals(true, ris.contains(object).check());
    }

    @Test
    public void testInNotInitLVarInitRisTrue1(){
        LVar y = new LVar();
        LVar x = new LVar();
        LSet domain = LSet.empty().ins("something").ins("somethingElse").ins("somethingDifferent");

        Ris ris = new Ris(x, domain, x.neq("somethingDifferent"));
        Solver solver = new Solver();
        solver.add(ris.contains(y));
        assertEquals(true, solver.check());
        assertEquals("somethingElse", y.getValue());
        assertTrue(solver.nextSolution());
        assertEquals("something", y.getValue());
        assertEquals(false, solver.nextSolution());
    }

    @Test
    public void testInNotInitLVarInitRisTrue2(){
        LVar y = new LVar();
        LVar x = new LVar();
        LSet domain = new LSet().ins("something").ins("somethingElse").ins("somethingDifferent");

        Ris ris = new Ris(x, domain, x.neq("somethingDifferent"));
        Solver solver = new Solver();
        solver.add(ris.contains(y));
        assertEquals(true, solver.check());
        assertEquals("somethingElse", y.getValue());
        assertTrue(solver.nextSolution());
        assertEquals("something", y.getValue());
        assertEquals(true, solver.nextSolution());
        assertNull(y.getValue());
    }

    @Test
    public void testInNotInitLVarInitRisFalse1(){
        LVar y = new LVar();
        LVar x = new LVar();

        LSet domain = LSet.empty().ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, x.eq("a").and(x.eq("b")));
        assertFalse(y.in(ris).check());
    }

    @Test
    public void testInNotInitLVarInitRisFalse2(){
        LVar y = new LVar();
        LVar x = new LVar();

        LSet domain = LSet.empty().ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, Constraint.falsec());
        assertFalse(y.in(ris).check());
    }

    @Test
    public void testInNotInitLVarInitRisFalse3(){
        LVar y = new LVar();
        LVar x = new LVar();

        LSet domain = new LSet().ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, x.eq("a").and(x.eq("b")));
        assertFalse(y.in(ris).check());
    }

    @Test
    public void testInNotInitLVarInitRisFalse4(){
        LVar y = new LVar();
        LVar x = new LVar();

        LSet domain = new LSet().ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, Constraint.falsec());
        assertFalse(y.in(ris).check());
    }

    @Test
    public void testInNotInitLVarInitRisFalse5(){
        LVar y = new LVar();
        LVar x = new LVar();

        LSet domain = LSet.empty().ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, x.neq("something").and(x.neq("somethingElse").and(x.neq("somethingDifferent"))));
        assertFalse(y.in(ris).check());
    }

    @Test
    public void testInInitLVarInitRisTrue1(){
        LVar y = new LVar("y","something");
        LVar x = new LVar();

        LSet domain = LSet.empty().ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, x.neq("somethingDifferent"));
        assertTrue(y.in(ris).check());
    }

    @Test
    public void testInInitLVarInitRisTrue2(){
        LVar y = new LVar("y","somethingElse");
        LVar x = new LVar();

        LSet domain = LSet.empty().ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, x.neq("somethingDifferent"));
        assertTrue(y.in(ris).check());
    }

    @Test
    public void testInInitLVarInitRisTrue3(){
        LVar y = new LVar("y","something");
        LVar x = new LVar();

        LSet domain = new LSet().ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, x.neq("somethingDifferent"));
        assertTrue(y.in(ris).check());
    }

    @Test
    public void testInInitLVarInitRisTrue4(){
        LVar y = new LVar("y","somethingElse");
        LVar x = new LVar();

        LSet domain = new LSet().ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, x.neq("somethingDifferent"));
        assertTrue(y.in(ris).check());
    }
    @Test
    public void testInInitLVarInitRisTrue5(){
        LVar y = new LVar("y","somethingElse(r)");
        LVar x = new LVar();

        LSet domain = new LSet().ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, x.neq("somethingDifferent"));
        assertTrue(y.in(ris).check());
        assertEquals("{somethingDifferent,somethingElse,something,somethingElse(r)/_?}", domain.toString());
    }

    @Test
    public void testInInitLVarInitRisTrue6(){
        LVar y = new LVar("y","somethingElse(r)");
        LVar x = new LVar();

        LSet domain = new LSet().ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, x.neq("somethingDifferent"));
        assertTrue(y.in(ris).check());
        assertEquals("{somethingDifferent,somethingElse,something,somethingElse(r)/_?}", domain.toString());
    }

    @Test
    public void testInInitLVarInitRisFalse1(){
        LVar y = new LVar("y","somethingElse(r)");
        LVar x = new LVar();

        LSet domain = LSet.empty().ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, x.neq("somethingDifferent"));
        assertFalse(y.in(ris).check());
    }

    @Test
    public void testInInitLVarInitRisFalse2(){
        LVar y = new LVar("y","somethingDifferent");
        LVar x = new LVar();

        LSet domain = new LSet().ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, x.neq("somethingDifferent"));
        assertFalse(y.in(ris).check());
    }

    @Test
    public void testInInitLVarInitRisFalse3(){
        LVar y = new LVar("y","somethingElse(r)");
        LVar x = new LVar();

        LSet domain = new LSet().ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, x.neq("somethingDifferent").and(x.neq("somethingElse(r)")));
        assertFalse(y.in(ris).check());
    }

    @Test
    public void testInInitLVarNotInitRisTrue1(){
        LVar y = new LVar("y","something");
        LVar x = new LVar();

        LSet domain = new LSet();
        Ris ris = new Ris(x, domain, x.neq("somethingDifferent"));
        assertTrue(y.in(ris).check());
        assertEquals("{something/_?}", domain.toString());
    }

    @Test
    public void testInInitLVarNotInitRisFalse1(){
        LVar y = new LVar("y","something");
        LVar x = new LVar();

        LSet domain = new LSet();
        Ris ris = new Ris(x, domain, x.neq("something"));
        assertFalse(y.in(ris).check());
    }

    /*-------------------------------------------------------------------
     _   _ _____ _   _   _______ ______  _____ _______ _____
    | \ | /_   _| \ | | /__   __|  ____|/ ____/__   __/ ____|
    |  \| | | | |  \| |    | |  | /__  | (___    | | | (___
    | . ` | | | | . ` |    | |  |  __|  \___ \   | |  \___ \
    | |\  /_| /_| |\  |    | |  | /____ ____) |  | |  ____) |
    /_| \_/_____/_| \_|    /_|  /______/_____/   /_| /_____/
    -------------------------------------------------------------------*/

    @Test
    public void testNinObjectEmptyRisTrue1(){
        Ris ris = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        assertEquals(true, ris.ncontains("something").check());
    }

    @Test
    public void testNinNotInitLVarEmptyRisTrue1(){
        Ris ris = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        LVar lVar = new LVar();
        assertEquals(true, lVar.nin(ris).check());
    }

    @Test
    public void testNinInitLVarEmptyRisTrue1(){
        Ris ris = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        LVar lVar = new LVar("something");
        assertEquals(true, lVar.nin(ris).check());
    }

    @Test
    public void testNinObjectInitRisFalse1(){
        Object object = "something";

        LVar x = new LVar();
        LSet domain = LSet.empty().ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, x.neq("somethingDifferent"));

        assertEquals(false, ris.ncontains(object).check());
    }

    @Test
    public void testNinObjectInitRisFalse2(){
        Object object = "something";

        LVar x = new LVar();
        LSet domain = new LSet().ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, x.neq("somethingDifferent"));

        assertEquals(false, ris.ncontains(object).check());
    }

    @Test
    public void testNinNotInitLVarInitRisTrue1(){
        LVar y = new LVar("y");
        LVar x = new LVar();
        LSet domain = LSet.empty().ins("something").ins("somethingElse").ins("somethingDifferent");

        Ris ris = new Ris(x, domain, x.neq("somethingDifferent"));
        Solver solver = new Solver();
        solver.add(ris.ncontains(y));
        assertEquals(true, solver.check());
        assertEquals("_y neq somethingElse AND _y neq something", solver.getConstraint().toString());
    }

    @Test
    public void testNinNotInitLVarInitRisTrue2(){
        LVar y = new LVar();
        LVar x = new LVar();
        LSet domain = new LSet("Rest").ins("something").ins("somethingElse").ins("somethingDifferent");

        Ris ris = new Ris(x, domain, x.neq("somethingDifferent"));
        Solver solver = new Solver();
        solver.add(ris.ncontains(y));
        assertEquals(true, solver.check());
        assertEquals("_? nin { _? : _Rest | _? neq somethingDifferent @ _? } AND _? neq somethingElse AND _? neq something",
                            solver.getConstraint().toString());
    }

    @Test
    public void testNinNotInitLVarInitRisTrue3(){
        LVar y = new LVar();
        LVar x = new LVar();

        LSet domain = LSet.empty().ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, x.eq("a").and(x.eq("b")));
        assertTrue(y.nin(ris).check());
    }

    @Test
    public void testNinNotInitLVarInitRisTrue4(){
        LVar y = new LVar();
        LVar x = new LVar();

        LSet domain = LSet.empty().ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, Constraint.falsec());
        assertTrue(y.nin(ris).check());
    }

    @Test
    public void testInNotInitLVarInitRisTrue5(){
        LVar y = new LVar();
        LVar x = new LVar();

        LSet domain = new LSet().ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, x.eq("a").and(x.eq("b")));
        assertFalse(y.in(ris).check());
    }

    @Test
    public void testNinNotInitLVarInitRisTrue5(){
        LVar y = new LVar();
        LVar x = new LVar();

        LSet domain = new LSet().ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, Constraint.falsec());
        assertTrue(y.nin(ris).check());
    }

    @Test
    public void testNinNotInitLVarInitRisTrue6(){
        LVar y = new LVar();
        LVar x = new LVar();

        LSet domain = LSet.empty().ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, x.neq("something").and(x.neq("somethingElse").and(x.neq("somethingDifferent"))));
        assertTrue(y.nin(ris).check());
    }

    @Test
    public void testNinInitLVarInitRisFalse1(){
        LVar y = new LVar("y","something");
        LVar x = new LVar();

        LSet domain = LSet.empty().ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, x.neq("somethingDifferent"));
        assertFalse(y.nin(ris).check());
    }

    @Test
    public void testNinInitLVarInitRisFalse2(){
        LVar y = new LVar("y","somethingElse");
        LVar x = new LVar();

        LSet domain = LSet.empty().ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, x.neq("somethingDifferent"));
        assertFalse(y.nin(ris).check());
    }

    @Test
    public void testNinInitLVarInitRisFalse3(){
        LVar y = new LVar("y","something");
        LVar x = new LVar();

        LSet domain = new LSet().ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, x.neq("somethingDifferent"));
        assertFalse(y.nin(ris).check());
    }

    @Test
    public void testNinInitLVarInitRisFalse4(){
        LVar y = new LVar("y","somethingElse");
        LVar x = new LVar();

        LSet domain = new LSet().ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, x.neq("somethingDifferent"));
        assertFalse(y.nin(ris).check());
    }
    @Test
    public void testNinInitLVarInitRisFalse5(){
        LVar y = new LVar("y","somethingElse(r)");
        LVar x = new LVar();

        LSet domain = new LSet().ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, x.neq("somethingDifferent"));
        assertTrue(y.nin(ris).check());
    }

    @Test
    public void testNinInitLVarInitRisTrue1(){
        LVar y = new LVar("y","somethingElse(r)");
        LVar x = new LVar();

        LSet domain = new LSet("Rest").ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, x.neq("somethingDifferent"));

        Solver solver = new Solver();
        solver.add(y.nin(ris));
        assertTrue(solver.check());
        assertEquals("somethingElse(r) nin { _? : _Rest | _? neq somethingDifferent @ _? }", solver.getConstraint().toString());
    }

    @Test
    public void testNinInitLVarInitRisTrue2(){
        LVar y = new LVar("y","somethingElse(r)");
        LVar x = new LVar();

        LSet domain = LSet.empty().ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, x.neq("somethingDifferent"));
        assertTrue(y.nin(ris).check());
    }

    @Test
    public void testNinInitLVarInitRisTrue3(){
        LVar y = new LVar("y","somethingDifferent");
        LVar x = new LVar();

        LSet domain = new LSet("Rest").ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, x.neq("somethingDifferent"));
        Solver solver = new Solver();
        solver.add(y.nin(ris));
        assertTrue(solver.check());
        assertEquals("somethingDifferent nin { _? : _Rest | _? neq somethingDifferent @ _? }",
                                solver.getConstraint().toString());
    }

    @Test
    public void testNinInitLVarInitRisTrue4(){
        LVar y = new LVar("y","somethingElse(r)");
        LVar x = new LVar();

        LSet domain = new LSet().ins("something").ins("somethingElse").ins("somethingDifferent");
        Ris ris = new Ris(x, domain, x.neq("somethingDifferent").and(x.neq("somethingElse(r)")));
        Solver solver = new Solver();
        solver.add(y.nin(ris));
        assertTrue(solver.check());
        assertEquals("somethingElse(r) nin { _? : _? | _? neq somethingDifferent AND _? neq somethingElse(r) @ _? }", solver.getConstraint().toString());
    }

    @Test
    public void testNinInitLVarNotInitRisTrue1(){
        LVar y = new LVar("y","something");
        LVar x = new LVar();

        LSet domain = new LSet("Rest");
        Ris ris = new Ris(x, domain, x.neq("somethingDifferent"));
        Solver solver = new Solver();
        solver.add(y.nin(ris));
        assertTrue(solver.check());
        assertEquals("something nin { _? : _Rest | _? neq somethingDifferent @ _? }", solver.getConstraint().toString());
    }

    @Test
    public void testNinInitLVarNotInitRisTrue2(){
        LVar y = new LVar("y","something");
        LVar x = new LVar();

        LSet domain = new LSet("Rest");
        Ris ris = new Ris(x, domain, x.neq("something"));
        Solver solver = new Solver();
        solver.add(y.nin(ris));
        assertTrue(solver.check());
        assertEquals("something nin { _? : _Rest | _? neq something @ _? }", solver.getConstraint().toString());
    }

    /*-------------------------------------------------------------------------------------------------
      _____ _____  _____      _  ____ _____ _   _ _______   _______ ______  _____ _______ _____
     |  __ \_   _|/ ____|    | |/ __ \_   _| \ | /__   __| /__   __|  ____|/ ____/__   __/ ____|
     | |  | || | | (___      | | |  | || | |  \| |  | |       | |  | /__  | (___    | | | (___
     | |  | || |  \___ \ _   | | |  | || | | . ` |  | |       | |  |  __|  \___ \   | |  \___ \
     | /__| || /_ ____) | /__| | /__| || /_| |\  |  | |       | |  | /____ ____) |  | |  ____) |
     /_____/_____/_____/ \____/ \____/_____/_| \_|  /_|       /_|  /______/_____/   /_| /_____/
     -------------------------------------------------------------------------------------------------*/

    @Test
    public void testDisjEmptySetEmptyRis(){
        LSet set1 = LSet.empty();
        Ris set2 = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set1.disj(set2));
        assertTrue(solver.check());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjEmptySetInitRis1(){
        LSet set1 = LSet.empty();
        Ris set2 = new Ris(new LVar(), LSet.empty().ins(1), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set1.disj(set2));
        assertTrue(solver.check());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjEmptySetInitRis2(){
        LSet set1 = LSet.empty();
        Ris set2 = new Ris(new LVar(), new LSet().ins(1), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set1.disj(set2));
        assertTrue(solver.check());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjEmptySetNotInitRis(){
        LSet set1 = LSet.empty();
        Ris set2 = new Ris(new LVar(), new LSet(), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set1.disj(set2));
        assertTrue(solver.check());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjInitSetEmptyRis1(){
        LSet set1 = LSet.empty().ins(1);
        Ris set2 = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set1.disj(set2));
        assertTrue(solver.check());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjInitSetEmptyRis2(){
        LSet set1 = new LSet().ins(1);
        Ris set2 = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set1.disj(set2));
        assertTrue(solver.check());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjInitSetInitRis1(){
        LSet set1 = LSet.empty().ins(2);
        Ris set2 = new Ris(new LVar(), LSet.empty().ins(1), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set1.disj(set2));
        assertTrue(solver.check());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjInitSetInitRis2(){
        LSet set1 = new LSet().ins(2);
        Ris set2 = new Ris(new LVar(), new LSet().ins(1), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set1.disj(set2));
        assertTrue(solver.check());
        assertFalse(solver.nextSolution());
    }

    public void testDisjInitSetInitRis3(){
        LSet set1 = LSet.empty().ins(1);
        Ris set2 = new Ris(new LVar(), LSet.empty().ins(1), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set1.disj(set2));
        assertFalse(solver.check());
    }

    @Test
    public void testDisjInitSetInitRis4(){
        LSet set1 = new LSet().ins(1);
        Ris set2 = new Ris(new LVar(), new LSet().ins(1), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set1.disj(set2));
        assertFalse(solver.check());
    }

    @Test
    public void testDisjInitSetNotInitRis1(){
        LSet set1 = LSet.empty().ins(1);
        Ris set2 = new Ris(new LVar(), new LSet(), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set1.disj(set2));
        assertTrue(solver.check());
        assertEquals("1 nin { _? : _? | true @ _? }", solver.getConstraint().toString());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjInitSetNotInitRis2(){
        LSet set1 = new LSet().ins(1);
        Ris set2 = new Ris(new LVar(), new LSet(), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set1.disj(set2));
        assertTrue(solver.check());
        assertEquals("disj(_?,{ _? : _? | true @ _? }) AND 1 nin { _? : _? | true @ _? }", solver.getConstraint().toString());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjVarSetEmptyRis(){
        LSet set1 = new LSet();
        Ris set2 = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set1.disj(set2));
        assertTrue(solver.check());
        assertEquals("[]", solver.getConstraint().toString());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjVarSetInitRis1(){
        LSet set1 = new LSet();
        IntLVar x = new IntLVar();
        Ris set2 = new Ris(x, LSet.empty().ins(1), Constraint.truec(), x.sum(1));
        Solver solver = new Solver();
        solver.add(set1.disj(set2));
        assertTrue(solver.check());
        assertEquals("2 nin _?", solver.getConstraint().toString());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjVarSetInitRis2(){
        LSet set1 = new LSet();
        Ris set2 = new Ris(new LVar(), new LSet().ins(1), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set1.disj(set2));
        assertTrue(solver.check());
        assertEquals("disj(_?,{ _? : _? | true @ _? }) AND 1 nin _?", solver.getConstraint().toString());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjVarSetNotInitRis(){
        LSet set1 = new LSet();
        Ris set2 = new Ris(new LVar(), new LSet(), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set1.disj(set2));
        assertTrue(solver.check());
        assertEquals("disj(_?,{ _? : _? | true @ _? })", solver.getConstraint().toString());

        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjEmptyRisEmptyRis(){
        Ris set1 = new Ris(new IntLVar(), IntLSet.empty(), Constraint.truec());
        Ris set2 = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set1.disj(set2));
        assertTrue(solver.check());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjEmptyRisInitRis1(){
        Ris set1 = new Ris(new IntLVar(), IntLSet.empty(), Constraint.truec());
        Ris set2 = new Ris(new LVar(), LSet.empty().ins(1), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set1.disj(set2));
        assertTrue(solver.check());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjEmptyRisInitRis2(){
        Ris set1 = new Ris(new IntLVar(), IntLSet.empty(), Constraint.truec());
        Ris set2 = new Ris(new LVar(), new LSet().ins(1), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set1.disj(set2));
        assertTrue(solver.check());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjEmptyRisNotInitRis(){
        LSet set1 = LSet.empty();
        Ris set2 = new Ris(new LVar(), new LSet(), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set1.disj(set2));
        assertTrue(solver.check());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjInitRisEmptySet1(){
        LSet set1 = LSet.empty();
        Ris set2 = new Ris(new LVar(), LSet.empty().ins(1), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set2.disj(set1));
        assertTrue(solver.check());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjInitRisEmptySet2(){
        LSet set1 = LSet.empty();
        Ris set2 = new Ris(new LVar(), new LSet().ins(1), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set2.disj(set1));
        assertTrue(solver.check());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjInitRisInitSet1(){
        LSet set1 = LSet.empty().ins(2);
        Ris set2 = new Ris(new LVar(), LSet.empty().ins(1), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set2.disj(set1));
        assertTrue(solver.check());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjInitRisInitSet2(){
        LSet set1 = new LSet().ins(2);
        Ris set2 = new Ris(new LVar(), new LSet().ins(1), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set2.disj(set1));
        assertTrue(solver.check());
        assertFalse(solver.nextSolution());
    }

    public void testDisjInitRisInitSet3(){
        LSet set1 = LSet.empty().ins(1);
        Ris set2 = new Ris(new LVar(), LSet.empty().ins(1), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set2.disj(set1));
        assertFalse(solver.check());
    }

    @Test
    public void testDisjInitRisInitSet4(){
        LSet set1 = new LSet().ins(1);
        Ris set2 = new Ris(new LVar(), new LSet().ins(1), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set2.disj(set1));
        assertFalse(solver.check());
    }

    @Test
    public void testDisjInitRisVarSet1(){
        LSet set1 = new LSet();
        IntLVar x = new IntLVar();
        Ris set2 = new Ris(x, LSet.empty().ins(1), Constraint.truec(), x.sum(1));
        Solver solver = new Solver();
        solver.add(set2.disj(set1));
        assertTrue(solver.check());
        assertEquals("2 nin _?", solver.getConstraint().toString());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjInitRisVarSet2(){
        LSet set1 = new LSet();
        Ris set2 = new Ris(new LVar(), new LSet().ins(1), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set2.disj(set1));
        assertTrue(solver.check());
        assertEquals("disj({ _? : _? | true @ _? },_?) AND 1 nin _?", solver.getConstraint().toString());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjInitRisEmptyRis1(){
        Ris set1 = new Ris(new IntLVar(), IntLSet.empty(), Constraint.truec());
        Ris set2 = new Ris(new LVar(), LSet.empty().ins(1), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set2.disj(set1));
        assertTrue(solver.check());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjInitRisEmptyRis2(){
        Ris set1 = new Ris(new IntLVar(), IntLSet.empty(), Constraint.truec());
        Ris set2 = new Ris(new LVar(), new LSet().ins(1), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set2.disj(set1));
        assertTrue(solver.check());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjInitRisInitRis1(){
        IntLVar x = new IntLVar();
        IntLSet d1 = new IntLSet().ins(1).ins(2).ins(3).ins(0);
        Ris set1 = new Ris(x, d1, x.mod(2).eq(0), x.sum(1));

        IntLVar y = new IntLVar();
        IntLSet d2 = new IntLSet().ins(0).ins(1).ins(2).ins(3).ins(4).ins(5);
        Ris set2 = new Ris(y, d2, y.mod(2).eq(1), y.sum(1));

        Solver solver = new Solver();
        solver.add(set2.disj(set1));
        assertTrue(solver.check());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjInitRisInitRis2(){
        IntLVar x = new IntLVar();
        IntLSet d1 = new IntLSet().ins(1).ins(2).ins(3).ins(0);
        Ris set1 = new Ris(x, d1, x.mod(2).eq(0), x.sum(1));

        IntLVar y = new IntLVar();
        IntLSet d2 = new IntLSet().ins(0).ins(1).ins(2).ins(3).ins(4).ins(5);
        Ris set2 = new Ris(y, d2, y.mod(2).eq(1).or(y.mod(4).eq(1)), y.sum(1));

        Solver solver = new Solver();
        solver.add(set2.disj(set1));
        assertTrue(solver.check());
        int i = 1;
        while(solver.nextSolution())
            ++i;

        assertEquals(1, i);
    }

    @Test
    public void testDisjInitRisInitRis3(){
        IntLVar x = new IntLVar();
        IntLSet d1 = new IntLSet().ins(1).ins(2).ins(3).ins(0);
        Ris set1 = new Ris(x, d1, x.mod(2).eq(0), x.sum(1));

        IntLVar y = new IntLVar();
        IntLSet d2 = new IntLSet().ins(0).ins(1).ins(2).ins(4).ins(5);
        Ris set2 = new Ris(y, d2, y.mod(2).eq(1).or(y.eq(2)), y.sum(1));

        Solver solver = new Solver();
        solver.add(set2.disj(set1));
        assertFalse(solver.check());
    }

    @Test
    public void testDisjInitRisInitRis4(){
        IntLVar x = new IntLVar();
        IntLSet d1 = new IntLSet().ins(1).ins(2).ins(3).ins(0);
        Ris set1 = new Ris(x, d1, x.mod(2).eq(1), x.sum(1));

        IntLVar y = new IntLVar();
        IntLSet d2 = new IntLSet().ins(0).ins(1).ins(3).ins(4).ins(5);
        Ris set2 = new Ris(y, d2, y.mod(2).eq(0).or(y.eq(1)), y.sum(1));

        Solver solver = new Solver();
        solver.add(set2.disj(set1));
        assertFalse(solver.check());
    }

    @Test
    public void testDisjInitRisNotInitRis1(){
        IntLVar x = new IntLVar();
        IntLSet d1 = new IntLSet().ins(1).ins(2).ins(3).ins(0);
        Ris set1 = new Ris(x, d1, x.mod(2).eq(0), x.sum(1));

        IntLVar y = new IntLVar();
        IntLSet d2 = new IntLSet();
        Ris set2 = new Ris(y, d2, y.mod(2).eq(1), y.sum(1));

        Solver solver = new Solver();
        solver.add(set2.disj(set1));
        assertTrue(solver.check());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjInitRisNotInitRis2(){
        IntLVar x = new IntLVar();
        IntLSet d1 = new IntLSet().ins(1).ins(2).ins(3).ins(0);
        Ris set1 = new Ris(x, d1, x.mod(2).eq(0), x.sum(1));

        IntLVar y = new IntLVar();
        IntLSet d2 = new IntLSet();
        Ris set2 = new Ris(y, d2, y.mod(2).eq(1).or(y.mod(4).eq(1)), y.sum(1));

        Solver solver = new Solver();
        solver.add(set2.disj(set1));
        assertTrue(solver.check());
        int i = 1;
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjNotInitRisEmptySet(){
        LSet set1 = LSet.empty();
        Ris set2 = new Ris(new LVar(), new LSet(), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set2.disj(set1));
        assertTrue(solver.check());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjNotInitRisInitSet1(){
        LSet set1 = LSet.empty().ins(1);
        Ris set2 = new Ris(new LVar(), new LSet(), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set2.disj(set1));
        assertTrue(solver.check());
        assertEquals("1 nin { _? : _? | true @ _? }", solver.getConstraint().toString());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjNotInitRisInitSet2(){
        LSet set1 = new LSet().ins(1);
        Ris set2 = new Ris(new LVar(), new LSet(), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set2.disj(set1));
        assertTrue(solver.check());
        assertEquals("disj({ _? : _? | true @ _? },_?) AND 1 nin { _? : _? | true @ _? }", solver.getConstraint().toString());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjNotInitRisVarSet(){
        LSet set1 = new LSet();
        Ris set2 = new Ris(new LVar(), new LSet(), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set2.disj(set1));
        assertTrue(solver.check());
        assertEquals("disj({ _? : _? | true @ _? },_?)", solver.getConstraint().toString());

        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjNotInitRisEmptyRis(){
        LSet set1 = LSet.empty();
        Ris set2 = new Ris(new LVar(), new LSet(), Constraint.truec());
        Solver solver = new Solver();
        solver.add(set2.disj(set1));
        assertTrue(solver.check());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjNotInitRisInitRis1(){
        IntLVar x = new IntLVar();
        IntLSet d1 = new IntLSet().ins(1).ins(2).ins(3).ins(0);
        Ris set1 = new Ris(x, d1, x.mod(2).eq(0), x.sum(1));

        IntLVar y = new IntLVar();
        IntLSet d2 = new IntLSet();
        Ris set2 = new Ris(y, d2, y.mod(2).eq(1), y.sum(1));

        Solver solver = new Solver();
        solver.add(set1.disj(set2));
        assertTrue(solver.check());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjNotInitRisInitRis2(){
        IntLVar x = new IntLVar();
        IntLSet d1 = new IntLSet().ins(1).ins(2).ins(3).ins(0);
        Ris set1 = new Ris(x, d1, x.mod(2).eq(0), x.sum(1));

        IntLVar y = new IntLVar();
        IntLSet d2 = new IntLSet();
        Ris set2 = new Ris(y, d2, y.mod(2).eq(1).or(y.mod(4).eq(1)), y.sum(1));

        Solver solver = new Solver();
        solver.add(set1.disj(set2));
        assertTrue(solver.check());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testDisjNotInitRisNotInitRis(){
        IntLVar x = new IntLVar();
        IntLSet d1 = new IntLSet();
        Ris set1 = new Ris(x, d1, x.mod(2).eq(0), x.sum(1));

        IntLVar y = new IntLVar();
        IntLSet d2 = new IntLSet();
        Ris set2 = new Ris(y, d2, y.mod(2).eq(1).or(y.mod(4).eq(1)), y.sum(1));

        Solver solver = new Solver();
        solver.add(set1.disj(set2));
        assertTrue(solver.check());
        assertFalse(solver.nextSolution());
    }










    @Test
    public void testUnionEmptySetEmptyRisEmptySet01(){
        LSet a = LSet.empty();
        LSet b = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(a.union(b,c));

        assertTrue(solver.check());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testUnionEmptyRisEmptySetEmptySet01(){
        LSet a = LSet.empty();
        LSet b = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(b.union(a,c));

        assertTrue(solver.check());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testUnionEmptyRisEmptyRisEmptySet01(){
        LSet a = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        LSet b = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(a.union(b,c));

        assertTrue(solver.check());
        assertFalse(solver.nextSolution());
    }

    @Test
    public void testUnionInitClosedSetEmptyRisEmptySet01(){
        LSet a = LSet.mkSet(2);
        LSet b = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(a.union(b,c));

        assertFalse(solver.check());
    }

    @Test
    public void testUnionEmptyRisInitClosedSetEmptySet01(){
        LSet a = LSet.mkSet(2);
        LSet b = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(b.union(a,c));

        assertFalse(solver.check());
    }

    @Test
    public void testUnionInitOpenSetEmptyRisEmptySet01(){
        LSet a = new LSet().ins(new LVar()).ins(1);
        LSet b = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(a.union(b,c));

        assertFalse(solver.check());
    }

    @Test
    public void testUnionEmptyRisInitOpenSetEmptySet01(){
        LSet a = new LSet().ins(new LVar()).ins(1);
        LSet b = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(b.union(a,c));

        assertFalse(solver.check());
    }

    @Test
    public void testUnionNotInitSetEmptyRisEmptySet01(){
        LSet a = new LSet();
        LSet b = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(a.union(b,c));

        assertTrue(solver.check());
        assertFalse(solver.nextSolution());
        assertEquals(LSet.empty(), a);
    }

    @Test
    public void testUnionEmptyRisNotInitSetEmptySet01(){
        LSet a = new LSet();
        LSet b = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(b.union(a,c));

        assertTrue(solver.check());
        assertFalse(solver.nextSolution());
        assertEquals(LSet.empty(), a);
    }














    @Test
    public void testUnionEmptySetInitClosedRisEmptySetFalse01(){
        LSet a = LSet.empty();
        LSet b = new Ris(new LVar(), LSet.mkSet(3), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(a.union(b,c));

        assertFalse(solver.check());
    }

    @Test
    public void testUnionInitClosedRisEmptySetEmptySetFalse01(){
        LSet a = LSet.empty();
        LSet b = new Ris(new LVar(), LSet.mkSet(3), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(b.union(a,c));

        assertFalse(solver.check());
    }

    @Test
    public void testUnionEmptyRisInitClosedRisEmptySetFalse01(){
        LSet a = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        LSet b = new Ris(new LVar(), LSet.mkSet(3), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(a.union(b,c));

        assertFalse(solver.check());
    }

    @Test
    public void testUnionInitClosedSetInitClosedRisEmptySetFalse01(){
        LSet a = LSet.mkSet(2);
        LSet b = new Ris(new LVar(), LSet.mkSet(3), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(a.union(b,c));

        assertFalse(solver.check());
    }

    @Test
    public void testUnionInitClosedRisInitClosedSetEmptySetFalse01(){
        LSet a = LSet.mkSet(2);
        LSet b = new Ris(new LVar(), LSet.mkSet(3), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(b.union(a,c));

        assertFalse(solver.check());
    }

    @Test
    public void testUnionInitOpenSetInitClosedRisEmptySetFalse01(){
        LSet a = new LSet().ins(new LVar()).ins(1);
        LSet b = new Ris(new LVar(), LSet.mkSet(3), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(a.union(b,c));

        assertFalse(solver.check());
    }

    @Test
    public void testUnionInitClosedRisInitOpenSetEmptySetFalse01(){
        LSet a = new LSet().ins(new LVar()).ins(1);
        LSet b = new Ris(new LVar(), LSet.mkSet(3), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(b.union(a,c));

        assertFalse(solver.check());
    }

    @Test
    public void testUnionNotInitSetInitClosedRisEmptySetFalse01(){
        LSet a = new LSet();
        LSet b = new Ris(new LVar(), LSet.mkSet(3), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(a.union(b,c));

        assertFalse(solver.check());
    }

    @Test
    public void testUnionInitClosedRisNotInitSetEmptySetFalse01(){
        LSet a = new LSet();
        LSet b = new Ris(new LVar(), LSet.mkSet(3), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(b.union(a,c));

        assertFalse(solver.check());

    }

    @Test
    public void testUnionEmptySetInitClosedRisEmptySetTrue01(){
        LSet a = LSet.empty();
        LSet b = new Ris(new LVar(), LSet.mkSet(3), Constraint.falsec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(a.union(b,c));

        assertTrue(solver.check());
    }

    @Test
    public void testUnionInitClosedRisEmptySetEmptySetTrue01(){
        LSet a = LSet.empty();
        LSet b = new Ris(new LVar(), LSet.mkSet(3), Constraint.falsec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(b.union(a,c));

        assertTrue(solver.check());
    }

    @Test
    public void testUnionEmptyRisInitClosedRisEmptySetTrue01(){
        LSet a = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        LSet b = new Ris(new LVar(), LSet.mkSet(3), Constraint.falsec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(a.union(b,c));

        assertTrue(solver.check());
    }

    @Test
    public void testUnionNotInitSetInitClosedRisEmptySetTrue01(){
        LSet a = new LSet();
        LSet b = new Ris(new LVar(), LSet.mkSet(3), Constraint.falsec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(a.union(b,c));

        assertTrue(solver.check());
    }

    @Test
    public void testUnionInitClosedRisNotInitSetEmptySetTrue01(){
        LSet a = new LSet();
        LSet b = new Ris(new LVar(), LSet.mkSet(3), Constraint.falsec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(b.union(a,c));

        assertTrue(solver.check());
        assertEquals(LSet.empty(), a);
    }


















    @Test
    public void testUnionEmptySetInitOpenRisEmptySetFalse01(){
        LSet a = LSet.empty();
        LSet b = new Ris(new LVar(), new LSet().ins(new LVar()).ins(new LVar()).ins(1), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(a.union(b,c));

        assertFalse(solver.check());
    }

    @Test
    public void testUnionInitOpenRisEmptySetEmptySetFalse01(){
        LSet a = LSet.empty();
        LSet b = new Ris(new LVar(), new LSet().ins(new LVar()).ins(new LVar()).ins(1), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(b.union(a,c));

        assertFalse(solver.check());
    }

    @Test
    public void testUnionEmptyRisInitOpenRisEmptySetFalse01(){
        LSet a = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        LSet b = new Ris(new LVar(), new LSet().ins(new LVar()).ins(new LVar()).ins(1), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(a.union(b,c));

        assertFalse(solver.check());
    }

    @Test
    public void testUnionInitClosedSetInitOpenRisEmptySetFalse01(){
        LSet a = LSet.mkSet(2);
        LSet b = new Ris(new LVar(), new LSet().ins(new LVar()).ins(new LVar()).ins(1), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(a.union(b,c));

        assertFalse(solver.check());
    }

    @Test
    public void testUnionInitOpenRisInitClosedSetEmptySetFalse01(){
        LSet a = LSet.mkSet(2);
        LSet b = new Ris(new LVar(), new LSet().ins(new LVar()).ins(new LVar()).ins(1), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(b.union(a,c));

        assertFalse(solver.check());
    }

    @Test
    public void testUnionInitOpenSetInitOpenRisEmptySetFalse01(){
        LSet a = new LSet().ins(new LVar()).ins(1);
        LSet b = new Ris(new LVar(), new LSet().ins(new LVar()).ins(new LVar()).ins(1), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(a.union(b,c));

        assertFalse(solver.check());
    }

    @Test
    public void testUnionInitOpenRisInitOpenSetEmptySetFalse01(){
        LSet a = new LSet().ins(new LVar()).ins(1);
        LSet b = new Ris(new LVar(), new LSet().ins(new LVar()).ins(new LVar()).ins(1), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(b.union(a,c));

        assertFalse(solver.check());
    }

    @Test
    public void testUnionNotInitSetInitOpenRisEmptySetFalse01(){
        LSet a = new LSet();
        LSet b = new Ris(new LVar(), new LSet().ins(new LVar()).ins(new LVar()).ins(1), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(a.union(b,c));

        assertFalse(solver.check());
    }

    @Test
    public void testUnionInitOpenRisNotInitSetEmptySetFalse01(){
        LSet a = new LSet();
        LSet b = new Ris(new LVar(), new LSet().ins(new LVar()).ins(new LVar()).ins(1), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(b.union(a,c));

        assertFalse(solver.check());

    }

    @Test
    public void testUnionEmptySetInitOpenRisEmptySetTrue01(){
        LSet a = LSet.empty();
        LSet b = new Ris(new LVar(), new LSet().ins(new LVar()).ins(new LVar()).ins(1), Constraint.falsec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(a.union(b,c));

        assertTrue(solver.check());
    }

    @Test
    public void testUnionInitOpenRisEmptySetEmptySetTrue01(){
        LSet a = LSet.empty();
        LSet b = new Ris(new LVar(), new LSet().ins(new LVar()).ins(new LVar()).ins(1), Constraint.falsec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(b.union(a,c));

        assertTrue(solver.check());
    }

    @Test
    public void testUnionEmptyRisInitOpenRisEmptySetTrue01(){
        LSet a = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        LSet b = new Ris(new LVar(), new LSet().ins(new LVar()).ins(new LVar()).ins(1), Constraint.falsec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(a.union(b,c));

        assertTrue(solver.check());
    }

    @Test
    public void testUnionNotInitSetInitOpenRisEmptySetTrue01(){
        LSet a = new LSet();
        LSet b = new Ris(new LVar(), new LSet().ins(new LVar()).ins(new LVar()).ins(1), Constraint.falsec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(a.union(b,c));

        assertTrue(solver.check());
    }

    @Test
    public void testUnionInitOpenRisNotInitSetEmptySetTrue01(){
        LSet a = new LSet();
        LSet b = new Ris(new LVar(), new LSet().ins(new LVar()).ins(new LVar()).ins(1), Constraint.falsec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(b.union(a,c));

        assertTrue(solver.check());
        assertEquals(LSet.empty(), a);
    }




































    @Test
    public void testUnionEmptySetNotInitRisEmptySetTrue01(){
        LSet a = LSet.empty();
        LSet b = new Ris(new LVar(), new LSet(), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(a.union(b,c));

        assertTrue(solver.check());
    }

    @Test
    public void testUnionNotInitRisEmptySetEmptySetTrue01(){
        LSet a = LSet.empty();
        LSet b = new Ris(new LVar(), new LSet(), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(b.union(a,c));

        assertTrue(solver.check());
    }

    @Test
    public void testUnionEmptyRisNotInitRisEmptySetTrue01(){
        LSet a = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        LSet b = new Ris(new LVar(), new LSet(), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(a.union(b,c));

        assertTrue(solver.check());
    }

    @Test
    public void testUnionInitClosedSetNotInitRisEmptySetFalse01(){
        LSet a = LSet.mkSet(2);
        LSet b = new Ris(new LVar(), new LSet(), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(a.union(b,c));

        assertFalse(solver.check());
    }

    @Test
    public void testUnionNotInitRisInitClosedSetEmptySetFalse01(){
        LSet a = LSet.mkSet(2);
        LSet b = new Ris(new LVar(), new LSet(), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(b.union(a,c));

        assertFalse(solver.check());
    }

    @Test
    public void testUnionInitOpenSetNotInitRisEmptySetFalse01(){
        LSet a = new LSet().ins(new LVar()).ins(1);
        LSet b = new Ris(new LVar(), new LSet(), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(a.union(b,c));

        assertFalse(solver.check());
    }

    @Test
    public void testUnionNotInitRisInitOpenSetEmptySetFalse01(){
        LSet a = new LSet().ins(new LVar()).ins(1);
        LSet b = new Ris(new LVar(), new LSet(), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(b.union(a,c));

        assertFalse(solver.check());
    }

    @Test
    public void testUnionNotInitSetNotInitRisEmptySetTrue01(){
        LSet a = new LSet();
        LSet b = new Ris(new LVar(), new LSet(), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(a.union(b,c));

        assertTrue(solver.check());
    }

    @Test
    public void testUnionNotInitRisNotInitSetEmptySetTrue01(){
        LSet a = new LSet();
        LSet b = new Ris(new LVar(), new LSet(), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(b.union(a,c));

        assertTrue(solver.check());

    }

    @Test
    public void testUnionNotInitSetNotInitRisRisEmptySetTrue01(){
        LSet a = new LSet();
        LSet b = new Ris(new LVar(), new LSet(), Constraint.truec());
        LSet c = LSet.empty();

        Solver solver = new Solver();
        solver.add(a.union(b,c));

        assertTrue(solver.check());
        assertEquals(LSet.empty(), a);
    }

    @Test
    public void testUnionOpenRisOpenRisOpenRisTrue01() {
        IntLSet d1 = new IntLSet().ins(1).ins(new IntLVar("x1"));
        IntLSet d2 = new IntLSet().ins(2).ins(new IntLVar("x3"));
        IntLSet d3 = new IntLSet().ins(new IntLVar("x5"));

        IntLVar ct = new IntLVar("ct");
        Ris ris1 = new Ris(ct, d1, ct.lt(10), ct.mul(2).sum(1));
        Ris ris2 = new Ris(ct, d2, ct.gt(3), ct.sum(1));
        Ris ris3 = new Ris(ct, d3, ct.gt(2).and(ct.lt(8)), ct.sub(1));

        Solver solver = new Solver();
        solver.add(ris1.union(ris2,ris3));
        assertTrue(solver.check());
        d1.output();
        d2.output();
        d3.output();
        assertEquals("{1..2/_?}", d1.toString()); // ris1 = {3, 5, ...
        assertEquals("{2, 5/_?}", d2.toString()); // ris2 = {3, 6, ...
        assertEquals("{4, 6..7/_?}", d3.toString()); // ris3 = {3, 6, 5, ...}

    }

    @Test
    public void testUnionOpenRisOpenRisOpenRisFalse01() {
        IntLSet d1 = new IntLSet().ins(1).ins(new IntLVar("x1"));
        IntLSet d2 = new IntLSet().ins(2).ins(new IntLVar("x3"));
        IntLSet d3 = new IntLSet().ins(new IntLVar("x5")).ins(3);

        IntLVar ct = new IntLVar("ct");
        Ris ris1 = new Ris(ct, d1, ct.lt(10), ct.mul(2).sum(1));
        Ris ris2 = new Ris(ct, d2, ct.gt(3), ct.sum(1));
        Ris ris3 = new Ris(ct, d3, ct.gt(2).and(ct.lt(8)), ct.sub(1));

        Solver solver = new Solver();
        solver.add(ris1.union(ris2,ris3));
        assertFalse(solver.check());
    }


































    /************************************************************************************************************************
      _____ _   _ _______ ______ _____   _____ ______ _____ _______ _____ ____  _   _   _______ ______  _____ _______ _____
     /_   _| \ | /__   __|  ____|  __ \ / ____|  ____/ ____/__   __/_   _/ __ \| \ | | /__   __|  ____|/ ____/__   __/ ____|
       | | |  \| |  | |  | /__  | /__) | (___ | /__ | |       | |    | || |  | |  \| |    | |  | /__  | (___    | | | (___
       | | | . ` |  | |  |  __| |  _  / \___ \|  __|| |       | |    | || |  | | . ` |    | |  |  __|  \___ \   | |  \___ \
      _| /_| |\  |  | |  | /____| | \ \ ____) | /___| /____   | |   _| || /__| | |\  |    | |  | /____ ____) |  | |  ____) |
     /_____/_| \_|  /_|  /______/_|  \_\_____//______\_____|  /_|  /_____\____//_| \_|    /_|  /______/_____/   /_| /_____/

     ***********************************************************************************************************************/

    public void testInters01True(){
        IntLVar x = new IntLVar("x");
        IntLSet s = new IntLSet(0,5);
        Ris ris1 = new Ris(x, s, x.lt(4), x.sum(1));
        Ris ris2 = new Ris(x, s, x.gt(2), x.sum(1));
        Ris ris3 = new Ris(x, s, x.gt(2).and(x.lt(4)), x.sum(1));

        assertTrue(ris1.inters(ris2,ris3).check());
    }

    public void testInters01False(){
        IntLVar x = new IntLVar("x");
        IntLSet s = new IntLSet(0,5);
        Ris ris1 = new Ris(x, s, x.lt(4), x.sum(1));
        Ris ris2 = new Ris(x, s, x.gt(2), x.sum(1));
        Ris ris3 = new Ris(x, s, x.gt(2).and(x.lt(4)), x.sum(2));

        assertFalse(ris1.inters(ris2,ris3).check());
    }


    /************************************************************************************************************
      _____ _____ ______ ______ ______ _____  ______ _   _  _____ ______   _______ ______  _____ _______ _____
     |  __ \_   _|  ____|  ____|  ____|  __ \|  ____| \ | |/ ____|  ____| /__   __|  ____|/ ____/__   __/ ____|
     | |  | || | | /__  | /__  | /__  | /__) | /__  |  \| | |    | /__       | |  | /__  | (___    | | | (___
     | |  | || | |  __| |  __| |  __| |  _  /|  __| | . ` | |    |  __|      | |  |  __|  \___ \   | |  \___ \
     | /__| || /_| |    | |    | /____| | \ \| /____| |\  | /____| /____     | |  | /____ ____) |  | |  ____) |
     /_____/_____/_|    /_|    /______/_|  \_\______/_| \_|\_____/______|    /_|  /______/_____/   /_| /_____/

     ***********************************************************************************************************/

    public void testDiff01True(){
        IntLVar x = new IntLVar("x");
        IntLSet s = new IntLSet(0,2);
        Ris ris1 = new Ris(x, s, x.lt(2), x.sum(1));
        Ris ris2 = new Ris(x, s, x.gt(0), x.sum(1));
        Ris ris3 = new Ris(x, s, x.le(0), x.sum(1));

        assertTrue(ris1.diff(ris2,ris3).check());
    }

    public void testDiff01False(){
        IntLVar x = new IntLVar("x");
        IntLSet s = new IntLSet(0,2);
        Ris ris1 = new Ris(x, s, x.lt(2), x.sum(1));
        Ris ris2 = new Ris(x, s, x.gt(0), x.sum(1));
        Ris ris3 = new Ris(x, s, x.le(1), x.sum(1));

        assertFalse(ris1.inters(ris2,ris3).check());
    }



    public void testLess01True(){
        IntLVar x = new IntLVar("x");
        IntLSet s = new IntLSet(0,5);
        IntLSet res = new IntLSet();
        Ris ris1 = new Ris(x, s, x.lt(4), x.sum(1));
        Ris ris2 = new Ris(x, res, x.lt(3), x.sum(1));

        assertTrue(ris1.less(4,ris2).check());
        assertEquals("{0..2/_?}", res.toString());
    }

    public void testLess01False(){
        IntLVar x = new IntLVar("x");
        IntLSet s = new IntLSet(0,5);
        Ris ris1 = new Ris(x, s, x.lt(4), x.sum(1));
        Ris ris2 = new Ris(x, s, x.lt(3), x.sum(1));

        assertFalse(ris1.less(2,ris2).check());
    }




    public void testForallElems01True(){
        IntLVar x = new IntLVar("x");
        IntLSet s = new IntLSet(-3,3);
        Ris ris = new Ris(x, s, x.lt(4), x.mul(2));

        IntLVar x1 = new IntLVar("x1");
        assertTrue(ris.forallElems(x1, x1.mod(2).eq(0)).check());
    }

    public void testForallElems01False(){
        IntLVar x = new IntLVar("x");
        IntLSet s = new IntLSet(-3,3);
        Ris ris = new Ris(x, s, x.lt(4), x.sum(2));

        IntLVar x1 = new IntLVar("x1");
        assertFalse(ris.forallElems(x1, x1.mod(2).eq(0)).check());
    }


    /*-----------------------------------------------------------------
       _____ _____ ____________   _______ ______  _____ _______ _____
      / ____/_   _/___  /  ____| /__   __|  ____|/ ____/__   __/ ____|
     | (___   | |    / /| /__       | |  | /__  | (___    | | | (___
      \___ \  | |   / / |  __|      | |  |  __|  \___ \   | |  \___ \
      ____) /_| /_ / /__| /____     | |  | /____ ____) |  | |  ____) |
     /_____//_____/_____/______|    /_|  /______/_____/   /_| /_____/
     -----------------------------------------------------------------*/

    public void testSizeEmptyNotInit(){
        IntLVar x = new IntLVar("x");
        IntLSet d = IntLSet.empty();
        Ris ris = new Ris(x, d, x.mod(2).eq(0), x.sum(1));
        IntLVar k = new IntLVar("k");

        Solver solver = new Solver();
        solver.add(ris.size(k));

        assertTrue(solver.check());
        assertEquals("[]", solver.getConstraint().toString());
        assertTrue(k.isBound());
        assertEquals((Integer) 0, k.getValue());
        assertFalse(solver.nextSolution());
    }

    public void testSizeEmptyInitZero(){
        IntLVar x = new IntLVar("x");
        IntLSet d = IntLSet.empty();
        Ris ris = new Ris(x, d, x.mod(2).eq(0), x.sum(1));
        IntLVar k = new IntLVar("k", 0);

        Solver solver = new Solver();
        solver.add(ris.size(k));

        assertTrue(solver.check());
        assertEquals("[]", solver.getConstraint().toString());
        assertFalse(solver.nextSolution());
    }

    public void testSizeEmptyInitNegative(){
        IntLVar x = new IntLVar("x");
        IntLSet d = IntLSet.empty();
        Ris ris = new Ris(x, d, x.mod(2).eq(0), x.sum(1));
        IntLVar k = new IntLVar("k", -2);

        Solver solver = new Solver();
        solver.add(ris.size(k));

        assertFalse(solver.check());
    }

    public void testSizeEmptyInitPositive(){
        IntLVar x = new IntLVar("x");
        IntLSet d = IntLSet.empty();
        Ris ris = new Ris(x, d, x.mod(2).eq(0), x.sum(1));
        IntLVar k = new IntLVar("k", 2);

        Solver solver = new Solver();
        solver.add(ris.size(k));

        assertFalse(solver.check());
    }

    public void testSizeInitClosedNotInitTrue(){
        IntLVar x = new IntLVar("x");
        IntLSet d = IntLSet.mkIntSet(3);
        Ris ris = new Ris(x, d, x.mod(2).eq(0), x.sum(1));
        IntLVar k = new IntLVar("k");

        Solver solver = new Solver();
        solver.add(ris.size(k));

        Set<Integer> possibleSizes = new HashSet<>();
        solver.forEachSolution(i -> possibleSizes.add(k.getValue()));
        assertEquals(4, possibleSizes.size());
        for(int i = 0; i <= 3; ++i)
            assertTrue(possibleSizes.contains(i));
    }

    public void testSizeInitClosedInitZeroTrue(){
        IntLVar x = new IntLVar("x");
        IntLSet d = IntLSet.mkIntSet(4);
        Ris ris = new Ris(x, d, x.mod(2).eq(0), x.sum(1));
        IntLVar k = new IntLVar("k", 0);

        Solver solver = new Solver();
        solver.add(ris.size(k));

        assertTrue(solver.check());
    }

    public void testSizeInitClosedInitZeroFalse(){
        IntLVar x = new IntLVar("x");
        IntLSet d = IntLSet.mkIntSet(4);
        Ris ris = new Ris(x, d, Constraint.truec(), x.sum(1));
        IntLVar k = new IntLVar("k", 0);

        Solver solver = new Solver();
        solver.add(ris.size(k));

        assertFalse(solver.check());
    }

    public void testSizeInitClosedInitNegativeFalse(){
        IntLVar x = new IntLVar("x");
        IntLSet d = IntLSet.mkIntSet(4);
        Ris ris = new Ris(x, d, x.mod(2).eq(0), x.sum(1));
        IntLVar k = new IntLVar("k", -2);

        Solver solver = new Solver();
        solver.add(ris.size(k));

        assertFalse(solver.check());
    }

    public void testSizeInitClosedInitPositiveTrue(){
        IntLVar x = new IntLVar("x");
        IntLSet d = IntLSet.mkIntSet(4);
        Ris ris = new Ris(x, d, x.mod(2).eq(0), x.sum(1));
        IntLVar k = new IntLVar("k", 2);

        Solver solver = new Solver();
        solver.add(ris.size(k));
        solver.add(ris.eq(IntLSet.empty().ins(1).ins(3)));

        assertTrue(solver.check());
    }

    public void testSizeInitClosedInitPositiveFalse(){
        IntLVar x = new IntLVar("x");
        IntLSet d = IntLSet.mkIntSet(4);
        Ris ris = new Ris(x, d, x.mod(2).eq(0), x.sum(1));
        IntLVar k = new IntLVar("k", 2);

        Solver solver = new Solver();
        solver.add(ris.size(k));
        solver.add(ris.eq(IntLSet.empty().ins(1).ins(3).ins(5)));

        assertFalse(solver.check());
    }

    public void testSizeInitOpenNotInitTrue(){
        IntLVar x = new IntLVar("x");
        IntLSet d = new IntLSet().ins(new IntLVar()).ins(new IntLVar()).ins(new IntLVar());
        Ris ris = new Ris(x, d, x.mod(2).eq(0), x.sum(1));
        IntLVar k = new IntLVar("k");

        Solver solver = new Solver();
        solver.add(ris.size(k));

        Set<MultiInterval> possibleSizes = new HashSet<>();
        solver.forEachSolution(i -> possibleSizes.add(k.getDomain()));
        assertEquals(4, possibleSizes.size());
        for(int i = 0; i <= 3; ++i)
            assertTrue(possibleSizes.contains(new MultiInterval(i, MultiInterval.SUP)));
    }

    public void testSizeInitOpenInitZeroTrue(){
        IntLVar x = new IntLVar("x");
        IntLSet d = new IntLSet().ins(new IntLVar()).ins(new IntLVar()).ins(new IntLVar());
        Ris ris = new Ris(x, d, x.mod(2).eq(0), x.sum(1));
        IntLVar k = new IntLVar("k", 0);

        Solver solver = new Solver();
        solver.add(ris.size(k));

        assertTrue(solver.check());
    }

    public void testSizeInitOpenInitZeroFalse(){
        IntLVar x = new IntLVar("x");
        IntLSet d = new IntLSet().ins(new IntLVar()).ins(new IntLVar()).ins(new IntLVar());
        Ris ris = new Ris(x, d, Constraint.truec(), x.sum(1));
        IntLVar k = new IntLVar("k", 0);

        Solver solver = new Solver();
        solver.add(ris.size(k));

        assertFalse(solver.check());
    }

    public void testSizeInitOpenInitNegativeFalse(){
        IntLVar x = new IntLVar("x");
        IntLSet d = new IntLSet().ins(new IntLVar()).ins(new IntLVar()).ins(new IntLVar());
        Ris ris = new Ris(x, d, x.mod(2).eq(0), x.sum(1));
        IntLVar k = new IntLVar("k", -2);

        Solver solver = new Solver();
        solver.add(ris.size(k));

        assertFalse(solver.check());
    }

    public void testSizeInitOpenInitPositiveTrue(){
        IntLVar x = new IntLVar("x");
        IntLSet d = new IntLSet().ins(new IntLVar()).ins(new IntLVar()).ins(new IntLVar());
        Ris ris = new Ris(x, d, x.mod(2).eq(0), x.sum(1));
        IntLVar k = new IntLVar("k", 2);

        Solver solver = new Solver();
        solver.add(ris.size(k));
        solver.add(ris.eq(IntLSet.empty().ins(1).ins(3)));

        assertTrue(solver.check());
    }

    public void testSizeInitOpenInitPositiveFalse(){
        IntLVar x = new IntLVar("x");
        IntLSet d = new IntLSet().ins(new IntLVar()).ins(new IntLVar()).ins(new IntLVar());
        Ris ris = new Ris(x, d, x.mod(2).eq(0), x.sum(1));
        IntLVar k = new IntLVar("k", 2);

        Solver solver = new Solver();
        solver.add(ris.size(k));
        solver.add(ris.eq(IntLSet.empty().ins(1).ins(3).ins(5)));

        assertFalse(solver.check());
    }

    public void testSizeNotInitNotInitTrue(){
        IntLVar x = new IntLVar("x");
        IntLSet d = new IntLSet("D");
        Ris ris = new Ris(x, d, x.mod(2).eq(0), x.sum(1));
        IntLVar k = new IntLVar("k");

        Solver solver = new Solver();
        solver.add(ris.size(k));

        assertTrue(solver.check());
        assertTrue(new ConstraintStringComparator("size({ _x : _D | _? = 0 AND _? = _x mod 2 @ _?, _? = _x + 1 },_k)",
                solver.getConstraint()).areEqualInSameOrder());
        assertFalse(solver.nextSolution());
    }

    public void testSizeNotInitInitZeroTrue(){
        IntLVar x = new IntLVar("x");
        IntLSet d = new IntLSet("D");
        Ris ris = new Ris(x, d, x.mod(2).eq(0), x.sum(1));
        IntLVar k = new IntLVar("k", 0);

        Solver solver = new Solver();
        solver.add(ris.size(k));

        assertTrue(solver.check());
    }

    public void testSizeNotInitInitNegativeFalse(){
        IntLVar x = new IntLVar("x");
        IntLSet d = new IntLSet("D");
        Ris ris = new Ris(x, d, x.mod(2).eq(0), x.sum(1));
        IntLVar k = new IntLVar("k", -2);

        Solver solver = new Solver();
        solver.add(ris.size(k));

        assertFalse(solver.check());
    }

    public void testSizeNotInitInitPositiveTrue(){
        IntLVar x = new IntLVar("x");
        IntLSet d = new IntLSet("D");
        Ris ris = new Ris(x, d, x.mod(2).eq(0), x.sum(1));
        IntLVar k = new IntLVar("k", 2);

        Solver solver = new Solver();
        solver.add(ris.size(k));

        assertTrue(solver.check());
    }

    public void testSizeNotInitInitPositiveFalse(){
        IntLVar x = new IntLVar("x");
        IntLSet d = new IntLSet("D");
        Ris ris = new Ris(x, d, Constraint.falsec(), x.sum(1));
        IntLVar k = new IntLVar("k", 2);

        Solver solver = new Solver();
        solver.add(ris.size(k));

        assertFalse(solver.check());
    }

    public void testSizeClosedRisWithBigDomainNotInitTrue(){
        IntLVar x = new IntLVar("x");
        IntLSet d = IntLSet.empty();
        for(int i = 0; i < 1000; ++i)
            d = d.ins(i);
        Ris ris = new Ris(x, d, x.mod(2).eq(0), x.sum(1));
        IntLVar k = new IntLVar("k");

        Solver solver = new Solver();
        solver.add(ris.size(k));

        assertTrue(solver.check());
        assertTrue(k.isBound());
        assertEquals((Integer) 500, k.getValue());
    }

    public void testSizeClosedRisWithBigDomainInitTrue(){
        IntLVar x = new IntLVar("x");
        IntLSet d = IntLSet.empty();
        for(int i = 0; i < 1000; ++i)
            d = d.ins(i);
        Ris ris = new Ris(x, d, x.mod(2).eq(0), x.sum(1));
        IntLVar k = new IntLVar("k", 500);

        Solver solver = new Solver();
        solver.add(ris.size(k));

        assertTrue(solver.check());
    }

    public void testSizeClosedRisWithBigDomainInitFalse(){
        IntLVar x = new IntLVar("x");
        IntLSet d = IntLSet.empty();
        for(int i = 0; i < 1000; ++i)
            d = d.ins(i);
        Ris ris = new Ris(x, d, x.mod(2).eq(0), x.sum(1));
        IntLVar k = new IntLVar("k", 400);

        Solver solver = new Solver();
        solver.add(ris.size(k));

        assertFalse(solver.check());
    }

    public void testSizeOpenRisWithBigDomainNotInitTrue(){
        IntLVar x = new IntLVar("x");
        IntLSet d = new IntLSet("Tail");
        int n = 10;
        for(int i = 0; i < n; ++i)
            d = d.ins(i);
        Ris ris = new Ris(x, d, x.mod(2).eq(0), x.sum(1));
        IntLVar k = new IntLVar("k");

        Solver solver = new Solver();
        solver.add(ris.size(k));

        assertTrue(solver.check());
        assertFalse(k.isBound());
        assertEquals(new MultiInterval(n / 2, MultiInterval.SUP), k.getDomain());
    }

    public void testSizeOpenRisWithBigDomainInitFalse(){
        IntLVar x = new IntLVar("x");
        IntLSet d = new IntLSet("Tail");
        int n = 10;
        for(int i = 0; i < n; ++i)
            d = d.ins(i);
        Ris ris = new Ris(x, d, x.mod(2).eq(0), x.sum(1));
        IntLVar k = new IntLVar("k", n / 2 - 1);

        Solver solver = new Solver();
        solver.add(ris.size(k));

        assertFalse(solver.check());
    }

    public void testDummyVariableBoundExceptionThrown(){
        IntLVar x = new IntLVar();
        IntLSet dom = new IntLSet();
        IntLVar y = new IntLVar();
        y.setValue(5);

        assertThrows(InitLObjectException.class, ()->{Ris ris = new Ris(x,dom, Constraint.truec(), x, y);});
    }




}