package jsetl.test;

import jsetl.*;

import jsetl.exception.Failure;
import org.junit.Test;
import junit.framework.TestCase;

import static org.junit.jupiter.api.Assertions.assertThrows;

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//     Testing the solver class                       %
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

public class SolverTest extends TestCase {

	@Test
	public void
	testEmptyCS_clearStore() {
		Solver solver = new Solver();
		Constraint c1 = solver.getConstraint();
		assertEquals("[]",c1.toString());
		IntLVar x = new IntLVar("x");
		solver.add(x.neq(1));		
		solver.clearStore();
		Constraint c2 = solver.getConstraint();
		assertEquals("[]",c2.toString());
	}

	@Test
	public void
	testAdd() {
		Solver solver = new Solver();
		IntLVar x = new IntLVar("x");
		IntLVar y = new IntLVar("y");
		solver.add(x.neq(1).and(y.neq(2)).and(x.neq(y)));
		Constraint c1 = solver.getConstraint();
		solver.clearStore();
		solver.add(x.neq(1));
		solver.add(y.neq(2));
		solver.add(x.neq(y));
		Constraint c2 = solver.getConstraint();
		assertEquals(c1,c2);
	}
	
	@Test
	public void
	testNextSolution() 
	throws Failure {
		Solver solver = new Solver();
		LVar x = new LVar("x");
		LSet s = LSet.empty().ins(3).ins(2).ins(1).setName("s");
		solver.solve(x.in(s));
		assertEquals(x,1);
		assertTrue(solver.nextSolution());
		assertEquals(x,2);
		assertTrue(solver.nextSolution());
		assertEquals(x,3);
		assertFalse(solver.nextSolution());
		assertEquals(x,3);
		assertEquals(solver.getConstraint().toString(),"[]");
	}

	@Test
	public void
	testNextSolution2() 
	throws Failure {
		Solver solver = new Solver();
		LVar x = new LVar("x");
		LVar y = new LVar("y");
		LSet s = LSet.empty().ins(2).ins(1).setName("s");
		LSet r = LSet.empty().ins(x).ins(y).setName("r");
		solver.solve(s.neq(r));
		assertEquals(r.toString(),"{_y,_x}");
		assertEquals(solver.getConstraint().toString(),"1 neq _y AND 1 neq _x");
		assertTrue(solver.nextSolution());
		assertEquals(solver.getConstraint().toString(),"2 neq _y AND 2 neq _x");
		assertTrue(solver.nextSolution());
		assertEquals(solver.getConstraint().toString(),"_y neq 1 AND _y neq 2");
		assertTrue(solver.nextSolution());
		assertEquals(solver.getConstraint().toString(),"_x neq 1 AND _x neq 2");
		assertFalse(solver.nextSolution());  // no more solutions
	}

	@Test
	public void
	testNextSolution_noGood() 
	throws Failure {
		Solver solver = new Solver();
		LVar x = new LVar("x");
		LSet s = LSet.empty().ins(3).ins(2).ins(1).setName("s");
		solver.add(x.in(s));
		solver.nextSolution();
		solver.nextSolution();
		assertEquals(x.toString(),"_x");  // everything is unchanged
		assertEquals(solver.getConstraint().toString(),"_x in {1,2,3}"); 
	}
	
	@Test
	public void
	testSetof() 
	throws Failure {
		Solver solver = new Solver();
		LVar x = new LVar("x");
		LSet s = LSet.empty().ins(3).ins(2).ins(1).setName("s");
		solver.add(x.in(s));
		LSet r = solver.setof(x);
		assertEquals(x.toString(),"_x");
		assertEquals(r.toString(),"{1,2,3}");
		assertEquals(solver.getConstraint().toString(),"_x in {1,2,3}");
	}

	@Test
	public void
	testSetofEmpty() 
	throws Failure {
		Solver solver = new Solver();
		LVar x = new LVar("x");
		LSet s = LSet.empty().ins(2).ins(1).setName("s");
		solver.add(x.in(s));
		solver.add(x.neq(1).and(x.neq(2)));
		LSet r = solver.setof(x);
//		System.out.println(x);
//		System.out.println(r);
//		System.out.println(solver.getConstraint());
		assertEquals(x.toString(),"_x");
		assertEquals(r.toString(),"{}");
		assertEquals(solver.getConstraint().toString(),"_x in {1,2} AND _x neq 1 AND _x neq 2");
	}

	@Test
	public void
	testSetof2() 
	throws Failure {
		Solver solver = new Solver();
		LVar x = new LVar("x");
		LSet y = new LSet("y");
		LSet z = y.ins(4);
		LSet s = LSet.empty().ins(3).ins(2).ins(1).setName("s");
//		solver.add(x.in(s).and(x.nin(z)));
		LSet r = solver.setof(x,x.in(s).and(x.nin(z)));  // r = {x : x in {1,2,3} and x nin {4|y}}
		assertEquals(x.toString(),"_x");
		assertEquals(r.toString(),"{1,2,3}");
		assertEquals(y.toString(),"_y");
		assertEquals(z.toString(),"{4/_y}");
		assertEquals(solver.getConstraint().toString(),"_x in {1,2,3} AND _x nin {4/_y}");
	}

	@Test
	public void
	testSetof_interval() 
	throws Failure {
		Solver solver = new Solver();
		IntLVar x = new IntLVar("x");
		LSet r = solver.setof(x,x.dom(1,10).and(x.label()));  // r = {x : x in 1..10}
		assertEquals(x.toString(),"_x");
		assertEquals(r.toString(),"{1,2,3,4,5,6,7,8,9,10}");
		assertEquals(solver.getConstraint().toString(),"_x::[1..10] AND _x.label()");
	}


	@Test
	public void
	testRisStrictModeOnThrowingThrows1(){
		Solver solver = new Solver();
		LVar dummy = new LVar();
		LVar ct = new LVar();
		Ris ris = new Ris(ct, new LSet(), dummy.eq(dummy), ct, dummy);
		assertThrows(Solver.UnsafeRisException.class, () -> {solver.check(ris.neq(ris));});
	}

	@Test
	public void
	testRisStrictModeOnThrowingThrows2(){
		LVar dummy = new LVar();
		LVar ct = new LVar();
		Ris ris = new Ris(ct, new LSet(), dummy.eq(dummy), ct, dummy);
		assertThrows(Solver.UnsafeRisException.class, ris.neq(ris)::check);
	}

	@Test
	public void
	testRisStrictModeOnThrowingThrows3(){
		LVar dummy = new LVar();
		LVar ct = new LVar();
		Ris ris = new Ris(ct, new LSet(), dummy.eq(dummy), ct, dummy);
		assertThrows(Solver.UnsafeRisException.class, ris.neq(ris)::solve);
	}

	@Test
	public void
	testRisStrictModeOnThrowingThrows4(){
		LVar dummy = new LVar();
		LVar ct = new LVar();
		Ris ris = new Ris(ct, new LSet(), dummy.eq(dummy), ct, dummy);
		assertThrows(Solver.UnsafeRisException.class, ris.neq(ris)::test);
	}

	@Test
	public void
	testRisStrictModeOnThrowingNotThrows1(){
		Solver solver = new Solver();
		IntLVar ct = new IntLVar();
		Ris ris = new Ris(ct, new IntLSet(1,5), ct.sum(1).eq(ct.sum(1)).and(ct.label()), ct);
		assertFalse(solver.check(ris.neq(ris)));
		solver.check(ris.neq(ris));
		solver.showStore();
	}

	@Test
	public void
	testRisStrictModeOnThrowingNotThrows2(){
		IntLVar ct = new IntLVar();
		Ris ris = new Ris(ct, new IntLSet(1,5), ct.sum(1).eq(ct.sum(1)).and(ct.label()), ct);
		assertFalse(ris.neq(ris).check());
	}

	@Test
	public void
	testRisStrictModeOnThrowingNotThrows3(){
		IntLVar ct = new IntLVar();
		Ris ris = new Ris(ct, new IntLSet(1,5), ct.sum(1).eq(ct.sum(1)).and(ct.label()), ct);
		assertThrows(Failure.class, ris.neq(ris)::solve);
	}

	@Test
	public void
	testRisStrictModeOnThrowingNotThrows4(){
		IntLVar ct = new IntLVar();
		Ris ris = new Ris(ct, new IntLSet(1,5), ct.sum(1).eq(ct.sum(1)).and(ct.label()), ct);
		assertFalse(ris.neq(ris).test());
	}

	@Test
	public void
	testRisStrictModeOffWarningTrue1(){
		assertTrue(Solver.getUseStrictRisMode());
		assertFalse(Solver.getUnsafeRisUsageWarning());
		Solver.setUseStrictRisMode(false);
		assertFalse(Solver.getUnsafeRisUsageWarning());

		Solver solver = new Solver();
		LVar dummy = new LVar();
		LVar ct = new LVar();
		Ris ris = new Ris(ct, new LSet(), dummy.eq(dummy), ct, dummy);
		assertFalse(solver.check(ris.neq(ris)));

		assertTrue(Solver.peekUnsafeRisUsageWarning());
		assertTrue(Solver.peekUnsafeRisUsageWarning());
		assertTrue(Solver.getUnsafeRisUsageWarning());
		assertFalse(Solver.peekUnsafeRisUsageWarning());
		assertFalse(Solver.peekUnsafeRisUsageWarning());

		Solver.setUseStrictRisMode(true);
		assertTrue(Solver.getUseStrictRisMode());
	}

	@Test
	public void
	testRisStrictModeOffWarningTrue2(){
		assertTrue(Solver.getUseStrictRisMode());
		assertFalse(Solver.getUnsafeRisUsageWarning());
		Solver.setUseStrictRisMode(false);
		assertFalse(Solver.getUnsafeRisUsageWarning());

		LVar dummy = new LVar();
		LVar ct = new LVar();
		Ris ris = new Ris(ct, new LSet(), dummy.eq(dummy), ct, dummy);
		assertFalse(ris.neq(ris).check());

		assertTrue(Solver.peekUnsafeRisUsageWarning());
		assertTrue(Solver.peekUnsafeRisUsageWarning());
		assertTrue(Solver.getUnsafeRisUsageWarning());
		assertFalse(Solver.peekUnsafeRisUsageWarning());
		assertFalse(Solver.peekUnsafeRisUsageWarning());

		Solver.setUseStrictRisMode(true);
		assertTrue(Solver.getUseStrictRisMode());
	}

	@Test
	public void
	testRisStrictModeOffWarningTrue3(){
		assertTrue(Solver.getUseStrictRisMode());
		assertFalse(Solver.getUnsafeRisUsageWarning());
		Solver.setUseStrictRisMode(false);
		assertFalse(Solver.getUnsafeRisUsageWarning());

		LVar dummy = new LVar();
		LVar ct = new LVar();
		Ris ris = new Ris(ct, new LSet(), dummy.eq(dummy), ct, dummy);
		assertThrows(Failure.class, ris.neq(ris)::solve);

		assertTrue(Solver.peekUnsafeRisUsageWarning());
		assertTrue(Solver.peekUnsafeRisUsageWarning());
		assertTrue(Solver.getUnsafeRisUsageWarning());
		assertFalse(Solver.peekUnsafeRisUsageWarning());
		assertFalse(Solver.peekUnsafeRisUsageWarning());

		Solver.setUseStrictRisMode(true);
		assertTrue(Solver.getUseStrictRisMode());
	}

	@Test
	public void
	testRisStrictModeOffWarningTrue4(){
		assertTrue(Solver.getUseStrictRisMode());
		assertFalse(Solver.getUnsafeRisUsageWarning());
		Solver.setUseStrictRisMode(false);
		assertFalse(Solver.getUnsafeRisUsageWarning());

		LVar dummy = new LVar();
		LVar ct = new LVar();
		Ris ris = new Ris(ct, new LSet(), dummy.eq(dummy), ct, dummy);
		assertFalse(ris.neq(ris).test());

		assertTrue(Solver.peekUnsafeRisUsageWarning());
		assertTrue(Solver.peekUnsafeRisUsageWarning());
		assertTrue(Solver.getUnsafeRisUsageWarning());
		assertFalse(Solver.peekUnsafeRisUsageWarning());
		assertFalse(Solver.peekUnsafeRisUsageWarning());

		Solver.setUseStrictRisMode(true);
		assertTrue(Solver.getUseStrictRisMode());
	}

	@Test
	public void
	testRisStrictModeOffWarningFalse1(){
		assertTrue(Solver.getUseStrictRisMode());
		assertFalse(Solver.getUnsafeRisUsageWarning());
		Solver.setUseStrictRisMode(false);
		assertFalse(Solver.getUnsafeRisUsageWarning());

		Solver solver = new Solver();
		IntLVar ct = new IntLVar();
		Ris ris = new Ris(ct, new IntLSet(1,5), ct.sum(1).eq(ct.sum(1)).and(ct.label()), ct);
		assertFalse(solver.check(ris.neq(ris)));

		assertFalse(Solver.peekUnsafeRisUsageWarning());
		assertFalse(Solver.peekUnsafeRisUsageWarning());
		assertFalse(Solver.getUnsafeRisUsageWarning());
		assertFalse(Solver.peekUnsafeRisUsageWarning());
		assertFalse(Solver.peekUnsafeRisUsageWarning());

		Solver.setUseStrictRisMode(true);
		assertTrue(Solver.getUseStrictRisMode());
	}

	@Test
	public void
	testRisStrictModeOffWarningFalse2(){
		assertTrue(Solver.getUseStrictRisMode());
		assertFalse(Solver.getUnsafeRisUsageWarning());
		Solver.setUseStrictRisMode(false);
		assertFalse(Solver.getUnsafeRisUsageWarning());

		IntLVar ct = new IntLVar();
		Ris ris = new Ris(ct, new IntLSet(1,5), ct.sum(1).eq(ct.sum(1)).and(ct.label()), ct);
		assertFalse(ris.neq(ris).check());

		assertFalse(Solver.peekUnsafeRisUsageWarning());
		assertFalse(Solver.peekUnsafeRisUsageWarning());
		assertFalse(Solver.getUnsafeRisUsageWarning());
		assertFalse(Solver.peekUnsafeRisUsageWarning());
		assertFalse(Solver.peekUnsafeRisUsageWarning());

		Solver.setUseStrictRisMode(true);
		assertTrue(Solver.getUseStrictRisMode());
	}

	@Test
	public void
	testRisStrictModeOffWarningFalse3() throws Failure{
		assertTrue(Solver.getUseStrictRisMode());
		assertFalse(Solver.getUnsafeRisUsageWarning());
		Solver.setUseStrictRisMode(false);
		assertFalse(Solver.getUnsafeRisUsageWarning());

		IntLVar ct = new IntLVar();
		Ris ris = new Ris(ct, new IntLSet(1,5), ct.sum(1).eq(ct.sum(1)).and(ct.label()), ct);
		assertThrows(Failure.class, ris.neq(ris)::solve);

		assertFalse(Solver.peekUnsafeRisUsageWarning());
		assertFalse(Solver.peekUnsafeRisUsageWarning());
		assertFalse(Solver.getUnsafeRisUsageWarning());
		assertFalse(Solver.peekUnsafeRisUsageWarning());
		assertFalse(Solver.peekUnsafeRisUsageWarning());

		Solver.setUseStrictRisMode(true);
		assertTrue(Solver.getUseStrictRisMode());
	}

	@Test
	public void
	testRisStrictModeOffWarningFalse4(){
		assertTrue(Solver.getUseStrictRisMode());
		assertFalse(Solver.getUnsafeRisUsageWarning());
		Solver.setUseStrictRisMode(false);
		assertFalse(Solver.getUnsafeRisUsageWarning());

		IntLVar ct = new IntLVar();
		Ris ris = new Ris(ct, new IntLSet(1,5), ct.sum(1).eq(ct.sum(1)).and(ct.label()), ct);
		assertFalse(ris.neq(ris).test());

		assertFalse(Solver.peekUnsafeRisUsageWarning());
		assertFalse(Solver.peekUnsafeRisUsageWarning());
		assertFalse(Solver.getUnsafeRisUsageWarning());
		assertFalse(Solver.peekUnsafeRisUsageWarning());
		assertFalse(Solver.peekUnsafeRisUsageWarning());

		Solver.setUseStrictRisMode(true);
		assertTrue(Solver.getUseStrictRisMode());
	}

	public void
	testRisStrictModeSetResetsWarning1(){
		assertTrue(Solver.getUseStrictRisMode());
		assertFalse(Solver.getUnsafeRisUsageWarning());
		Solver.setUseStrictRisMode(false);
		assertFalse(Solver.getUnsafeRisUsageWarning());

		LVar dummy = new LVar();
		LVar ct = new LVar();
		Ris ris = new Ris(ct, new LSet(), dummy.eq(dummy), ct, dummy);
		assertFalse(ris.neq(ris).test());

		assertTrue(Solver.peekUnsafeRisUsageWarning());
		assertTrue(Solver.peekUnsafeRisUsageWarning());


		Solver.setUseStrictRisMode(true);
		assertTrue(Solver.getUseStrictRisMode());
		assertFalse(Solver.peekUnsafeRisUsageWarning());
		assertFalse(Solver.getUnsafeRisUsageWarning());
		assertFalse(Solver.peekUnsafeRisUsageWarning());
	}

	public void
	testRisStrictModeSetResetsWarning2(){
		assertTrue(Solver.getUseStrictRisMode());
		assertFalse(Solver.getUnsafeRisUsageWarning());
		Solver.setUseStrictRisMode(false);
		assertFalse(Solver.getUnsafeRisUsageWarning());

		LVar dummy = new LVar();
		LVar ct = new LVar();
		Ris ris = new Ris(ct, new LSet(), dummy.eq(dummy), ct, dummy);
		assertFalse(ris.neq(ris).test());

		assertTrue(Solver.peekUnsafeRisUsageWarning());
		assertTrue(Solver.peekUnsafeRisUsageWarning());


		Solver.setUseStrictRisMode(false);
		assertFalse(Solver.getUseStrictRisMode());
		assertFalse(Solver.peekUnsafeRisUsageWarning());
		assertFalse(Solver.getUnsafeRisUsageWarning());
		assertFalse(Solver.peekUnsafeRisUsageWarning());
		Solver.setUseStrictRisMode(true);
		assertTrue(Solver.getUseStrictRisMode());
	}

}
