package jsetl.test.utilities.comparing;


import jsetl.Constraint;

/**
 * This class is used to compare two constraint string representation and check whether they are equal or not without
 * taking into account the differences in the (automatically generated) numbers in variable names (as long as they differ
 * in a consistent way, i.e. a bijective function that maps each number from the first string into the corresponding number
 * in the second string exists). This check is done by normalizing both strings first, using an algorithm that maps
 * each string into their respcective normalized strings, which are equal if and only if the two strings are
 * the same except for the numbers in variable names.
 * @author Andrea Fois
 * @see ConstraintStringNormalizer
 */
public class ConstraintStringComparator {
    /**
     * Default character used to recognized variable name prefixes in constraint string representations.
     */
    public static char DEFAULT_VARIABLE_NAME_PREFIX = '_';

    /**
     * Character used to recognized variable name prefixes in constraint string representations.
     */
    private char variableNamePrefix = '_';

    /**
     * The first string in the comparison
     */
    private String constraintString1;

    /**
     * The second string in the comparison
     */
    private String constraintString2;

    /**
     * Constructs a new comparator using the given prefix character and constraint string representations
     * @param variableNamePrefix character used to recognize variable name prefixes
     * @param constraintString1 first string of the comparison
     * @param constraintString2 second string of the comparison
     */
    public ConstraintStringComparator(char variableNamePrefix, String constraintString1, String constraintString2){
        this.variableNamePrefix = variableNamePrefix;
        this.constraintString1 = constraintString1;
        this.constraintString2 = constraintString2;
    }

    /**
     * Constructs a new comparator using the given constraint string representations and default variable name prefix.
     * @param constraintString1 first string of the comparison
     * @param constraintString2 second string of the comparison
     */
    public ConstraintStringComparator(String constraintString1, String constraintString2){
        this(DEFAULT_VARIABLE_NAME_PREFIX, constraintString1, constraintString2);
    }

    /**
     * Constructs a new comparator using the given prefix character and constraint (string representations)
     * @param variableNamePrefix character used to recognize variable name prefixes
     * @param constraint1 constraint whose string representation is the first argument of the comparison
     * @param constraintString2 second string of the comparison
     */
    public ConstraintStringComparator(char variableNamePrefix, Constraint constraint1, String constraintString2){
        this(variableNamePrefix, constraint1.toString(), constraintString2);
    }

    /**
     * Constructs a new comparator using the given constraints (string representations) and default variable name prefix.
     * @param constraint1 constraint whose string representation is the first argument of the comparison
     * @param constraintString2 second string of the comparison
     */
    public ConstraintStringComparator(Constraint constraint1, String constraintString2){
        this(DEFAULT_VARIABLE_NAME_PREFIX, constraint1, constraintString2);
    }

    /**
     * Constructs a new comparator using the given prefix character and constraint (string representations)
     * @param variableNamePrefix character used to recognize variable name prefixes
     * @param constraintString1 first string of the comparison
     * @param constraint2 constraint whose string representation is the second argument of the comparison
     */
    public ConstraintStringComparator(char variableNamePrefix, String constraintString1, Constraint constraint2){
        this(variableNamePrefix, constraintString1, constraint2.toString());
    }

    /**
     * Constructs a new comparator using the given constraints (string representations) and default variable name prefix.
     * @param constraintString1 first string of the comparison
     * @param constraint2 constraint whose string representation is the second argument of the comparison
     */
    public ConstraintStringComparator(String constraintString1, Constraint constraint2){
        this(DEFAULT_VARIABLE_NAME_PREFIX, constraintString1, constraint2.toString());
    }

    /**
     * Constructs a new comparator using the given prefix character and constraints
     * @param variableNamePrefix character used to recognize variable name prefixes
     * @param constraint1 constraint whose string representation is the first argument of the comparison
     * @param constraint2 constraint whose string representation is the second argument of the comparison
     */
    public ConstraintStringComparator(char variableNamePrefix, Constraint constraint1, Constraint constraint2){
        this(variableNamePrefix, constraint1.toString(), constraint2.toString());
    }

    /**
     * Constructs a new comparator using the given constraints and default variable name prefix
     * @param constraint1 constraint whose string representation is the first argument of the comparison
     * @param constraint2 constraint whose string representation is the second argument of the comparison
     */
    public ConstraintStringComparator(Constraint constraint1, Constraint constraint2){
        this(DEFAULT_VARIABLE_NAME_PREFIX, constraint1, constraint2);
    }

    /**
     * Normalizes both <code>constraintString1</code> and <code>constraintString2</code> and compares them.
     * @return true if <code>constraintString1</code> and <code>constraintString2</code> are equal except for
     * numbers in variable names (which must differ in a consistent way), false otherwise.
     */
    public boolean areEqualInSameOrder(){
        String normalizedConstraintString1 = new ConstraintStringNormalizer(constraintString1).normalize();
        String normalizedConstraintString2 = new ConstraintStringNormalizer(constraintString2).normalize();

        return normalizedConstraintString1.equals(normalizedConstraintString2);
    }

}
